#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ethernet_qos_policy
short_description: Ethernet QoS Policy configuration for Cisco Intersight
description:
  - Manages Ethernet QoS Policy configuration on Cisco Intersight.
  - A policy to configure Quality of Service settings for Ethernet virtual interfaces on Cisco Intersight managed servers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/EthQosPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Ethernet QoS Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Ethernet QoS Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  mtu:
    description:
      - The Maximum Transmission Unit (MTU) or packet size that the virtual interface accepts.
      - The acceptable range is 1500-9000.
    type: int
    default: 1500
  cos:
    description:
      - Class of Service to be associated to the traffic on the virtual interface.
      - The acceptable range is 0-6.
    type: int
    default: 0
  priority:
    description:
      - The priority matching the System QoS specified in the fabric profile.
    type: str
    choices: ['Best Effort', 'FC', 'Platinum', 'Gold', 'Silver', 'Bronze']
    default: 'Best Effort'
  trust_host_cos:
    description:
      - Enables usage of the Class of Service provided by the operating system.
    type: bool
    default: false
  rate_limit:
    description:
      - The value in Mbps (0-10G/40G/100G depending on Adapter Model) to use for limiting the data rate on the virtual interface.
      - The range is between 0 and 100000.
    type: int
    default: 0
  burst:
    description:
      - The burst traffic, in bytes, allowed on the vNIC.
      - The range is between 1 to 1000000.
    type: int
    default: 10240
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create an Ethernet QoS Policy with default settings
  cisco.intersight.intersight_ethernet_qos_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "default-qos-policy"
    description: "Default Ethernet QoS policy"
    state: present

- name: Create an Ethernet QoS Policy with Platinum priority and trust host CoS
  cisco.intersight.intersight_ethernet_qos_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "platinum-qos-policy"
    description: "High priority Ethernet QoS policy"
    tags:
      - Key: "Environment"
        Value: "Production"
    mtu: 9000
    cos: 5
    priority: "Platinum"
    trust_host_cos: true
    rate_limit: 10000
    burst: 100000
    state: present

- name: Create an Ethernet QoS Policy with custom settings
  cisco.intersight.intersight_ethernet_qos_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "custom-qos-policy"
    description: "Custom Ethernet QoS policy for specific workload"
    mtu: 1500
    cos: 3
    priority: "Gold"
    trust_host_cos: false
    rate_limit: 5000
    burst: 50000
    state: present

- name: Delete an Ethernet QoS Policy
  cisco.intersight.intersight_ethernet_qos_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "custom-qos-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "platinum-qos-policy",
        "ObjectType": "vnic.EthQosPolicy",
        "Mtu": 9000,
        "Cos": 5,
        "Priority": "Platinum",
        "TrustHostCos": true,
        "RateLimit": 10000,
        "Burst": 100000,
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_input(module):
    """Validate input parameters"""
    mtu = module.params.get('mtu')
    if mtu is not None and (mtu < 1500 or mtu > 9000):
        module.fail_json(msg='MTU must be between 1500 and 9000')

    cos = module.params.get('cos')
    if cos is not None and (cos < 0 or cos > 6):
        module.fail_json(msg='CoS must be between 0 and 6')

    rate_limit = module.params.get('rate_limit')
    if rate_limit is not None and (rate_limit < 0 or rate_limit > 100000):
        module.fail_json(msg='Rate limit must be between 0 and 100000')

    burst = module.params.get('burst')
    if burst is not None and (burst < 1 or burst > 1000000):
        module.fail_json(msg='Burst must be between 1 and 1000000')


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        mtu=dict(type='int', default=1500),
        cos=dict(type='int', default=0),
        priority=dict(
            type='str',
            choices=['Best Effort', 'FC', 'Platinum', 'Gold', 'Silver', 'Bronze'],
            default='Best Effort'
        ),
        trust_host_cos=dict(type='bool', default=False),
        rate_limit=dict(type='int', default=0),
        burst=dict(type='int', default=10240),
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    validate_input(module)
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/vnic/EthQosPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        intersight.api_body.update({
            'Mtu': module.params['mtu'],
            'Cos': module.params['cos'],
            'Priority': module.params['priority'],
            'TrustHostCos': module.params['trust_host_cos'],
            'RateLimit': module.params['rate_limit'],
            'Burst': module.params['burst']
        })

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2013, Matthias Vogelgesang <matthias.vogelgesang@gmail.com>
# Copyright (c) 2014, Justin Lecher <jlec@gentoo.org>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: zypper_repository
author: "Matthias Vogelgesang (@matze)"
short_description: Add and remove Zypper repositories
description:
  - Add or remove Zypper repositories on SUSE and openSUSE.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    description:
      - A name for the repository. Not required when adding repofiles.
    type: str
  repo:
    description:
      - URI of the repository or full path of a C(.repo) file. Required when O(state=present).
    type: str
  state:
    description:
      - Whether the repository should exist or not.
      - A source string state.
    choices: ["absent", "present"]
    default: "present"
    type: str
  description:
    description:
      - A description of the repository.
    type: str
  disable_gpg_check:
    description:
      - Whether to disable GPG signature checking of all packages. Has an effect only if O(state=present).
      - Needs C(zypper) version >= 1.6.2.
      - When not specified, the value from the C(.repo) file is used if available, otherwise it defaults to V(false).
    type: bool
  autorefresh:
    description:
      - Enable autorefresh of the repository.
      - When not specified, the value from the C(.repo) file is used if available, otherwise it defaults to V(true).
    type: bool
    aliases: ["refresh"]
  priority:
    description:
      - Set priority of repository. Packages are always installed from the repository with the smallest priority number.
      - Needs C(zypper) version >= 1.12.25.
    type: int
  overwrite_multiple:
    description:
      - Overwrite multiple repository entries, if repositories with both name and URL already exist.
    type: bool
    default: false
  auto_import_keys:
    description:
      - Automatically import the gpg signing key of the new or changed repository.
      - Has an effect only if O(state=present). Has no effect on existing (unchanged) repositories or in combination with
        O(state=absent).
      - Implies O(runrefresh).
      - Only works with C(.repo) files if O(name) is given explicitly.
    type: bool
    default: false
  runrefresh:
    description:
      - Refresh the package list of the given repository.
      - Can be used with O(repo=*) to refresh all repositories.
    type: bool
    default: false
  enabled:
    description:
      - Set repository to enabled (or disabled).
      - When not specified, the value from the C(.repo) file is used if available, otherwise it defaults to V(true).
    type: bool


requirements:
  - "zypper >= 1.0  # included in openSUSE >= 11.1 or SUSE Linux Enterprise Server/Desktop >= 11.0"
  - python-xml
"""

EXAMPLES = r"""
- name: Add NVIDIA repository for graphics drivers
  community.general.zypper_repository:
    name: nvidia-repo
    repo: 'ftp://download.nvidia.com/opensuse/12.2'
    state: present

- name: Remove NVIDIA repository
  community.general.zypper_repository:
    name: nvidia-repo
    repo: 'ftp://download.nvidia.com/opensuse/12.2'
    state: absent

- name: Add python development repository
  community.general.zypper_repository:
    repo: 'http://download.opensuse.org/repositories/devel:/languages:/python/SLE_11_SP3/devel:languages:python.repo'

- name: Refresh all repos
  community.general.zypper_repository:
    repo: '*'
    runrefresh: true

- name: Add a repo and add its gpg key
  community.general.zypper_repository:
    repo: 'http://download.opensuse.org/repositories/systemsmanagement/openSUSE_Leap_42.1/'
    auto_import_keys: true

- name: Force refresh of a repository
  community.general.zypper_repository:
    repo: 'http://my_internal_ci_repo/repo'
    name: my_ci_repo
    state: present
    runrefresh: true
"""

import configparser
import traceback

XML_IMP_ERR = None
try:
    from xml.dom.minidom import parseString as parseXML

    HAS_XML = True
except ImportError:
    XML_IMP_ERR = traceback.format_exc()
    HAS_XML = False

from io import StringIO

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

REPO_OPTS = ["alias", "name", "priority", "enabled", "autorefresh", "gpgcheck"]


def _get_cmd(module, *args):
    """Combines the non-interactive zypper command with arguments/subcommands"""
    cmd = [module.get_bin_path("zypper", required=True), "--quiet", "--non-interactive"]
    cmd.extend(args)

    return cmd


def _parse_repos(module):
    """parses the output of zypper --xmlout repos and return a parse repo dictionary"""
    cmd = _get_cmd(module, "--xmlout", "repos")

    if not HAS_XML:
        module.fail_json(msg=missing_required_lib("python-xml"), exception=XML_IMP_ERR)
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    if rc == 0:
        repos = []
        dom = parseXML(stdout)
        repo_list = dom.getElementsByTagName("repo")
        for repo in repo_list:
            opts = {}
            for o in REPO_OPTS:
                opts[o] = repo.getAttribute(o)
            try:
                opts["url"] = repo.getElementsByTagName("url")[0].firstChild.data
            except IndexError:
                opts["url"] = repo.getAttribute("metalink")
            # A repo can be uniquely identified by an alias + url
            repos.append(opts)
        return repos
    # exit code 6 is ZYPPER_EXIT_NO_REPOS (no repositories defined)
    elif rc == 6:
        return []
    else:
        module.fail_json(msg=f'Failed to execute "{" ".join(cmd)}"', rc=rc, stdout=stdout, stderr=stderr)


def _repo_changes(module, realrepo, repocmp):
    "Check whether the 2 given repos have different settings."
    for k in repocmp:
        if repocmp[k] and k not in realrepo:
            return True

    for k, v in realrepo.items():
        if k in repocmp and repocmp[k]:
            valold = str(repocmp[k] or "")
            valnew = v or ""
            if k == "url":
                if "$releasever" in valold or "$releasever" in valnew:
                    cmd = ["rpm", "-q", "--qf", "%{version}", "-f", "/etc/os-release"]
                    rc, stdout, stderr = module.run_command(cmd, check_rc=True)
                    valnew = valnew.replace("$releasever", stdout)
                    valold = valold.replace("$releasever", stdout)
                if "$basearch" in valold or "$basearch" in valnew:
                    cmd = ["rpm", "-q", "--qf", "%{arch}", "-f", "/etc/os-release"]
                    rc, stdout, stderr = module.run_command(cmd, check_rc=True)
                    valnew = valnew.replace("$basearch", stdout)
                    valold = valold.replace("$basearch", stdout)
                valold, valnew = valold.rstrip("/"), valnew.rstrip("/")
            if valold != valnew:
                return True
    return False


def repo_exists(module, repodata, overwrite_multiple):
    """Check whether the repository already exists.

    returns (exists, mod, old_repos)
        exists: whether a matching (name, URL) repo exists
        mod: whether there are changes compared to the existing repo
        old_repos: list of matching repos
    """
    existing_repos = _parse_repos(module)

    # look for repos that have matching alias or url to the one searched
    repos = []
    for kw in ["alias", "url"]:
        name = repodata[kw]
        for oldr in existing_repos:
            if repodata[kw] == oldr[kw] and oldr not in repos:
                repos.append(oldr)

    if len(repos) == 0:
        # Repo does not exist yet
        return (False, False, None)
    elif len(repos) == 1:
        # Found an existing repo, look for changes
        has_changes = _repo_changes(module, repos[0], repodata)
        return (True, has_changes, repos)
    elif len(repos) >= 2:
        if overwrite_multiple:
            # Found two repos and want to overwrite_multiple
            return (True, True, repos)
        else:
            errmsg = f'More than one repo matched "{name}": "{repos}".'
            errmsg += " Use overwrite_multiple to allow more than one repo to be overwritten"
            module.fail_json(msg=errmsg)


def addmodify_repo(module, repodata, old_repos, zypper_version):
    "Adds the repo, removes old repos before, that would conflict."
    repo = repodata["url"]
    cmd = _get_cmd(module, "addrepo", "--check")
    if repodata["name"]:
        cmd.extend(["--name", repodata["name"]])

    # priority on addrepo available since 1.12.25
    # https://github.com/openSUSE/zypper/blob/b9b3cb6db76c47dc4c47e26f6a4d2d4a0d12b06d/package/zypper.changes#L327-L336
    if repodata["priority"]:
        if zypper_version >= LooseVersion("1.12.25"):
            cmd.extend(["--priority", str(repodata["priority"])])
        else:
            module.warn("Setting priority only available for zypper >= 1.12.25. Ignoring priority argument.")

    if repodata["enabled"] == "0":
        cmd.append("--disable")

    # gpgcheck available since 1.6.2
    # https://github.com/openSUSE/zypper/blob/b9b3cb6db76c47dc4c47e26f6a4d2d4a0d12b06d/package/zypper.changes#L2446-L2449
    # the default changed in the past, so don't assume a default here and show warning for old zypper versions
    if zypper_version >= LooseVersion("1.6.2"):
        if repodata["gpgcheck"] == "1":
            cmd.append("--gpgcheck")
        else:
            cmd.append("--no-gpgcheck")
    else:
        module.warn("Enabling/disabling gpgcheck only available for zypper >= 1.6.2. Using zypper default value.")

    if repodata["autorefresh"] == "1":
        cmd.append("--refresh")

    cmd.append(repo)

    if not repo.endswith(".repo"):
        cmd.append(repodata["alias"])

    if old_repos is not None:
        for oldrepo in old_repos:
            remove_repo(module, oldrepo["url"])

    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    return rc, stdout, stderr


def remove_repo(module, repo):
    "Removes the repo."
    cmd = _get_cmd(module, "removerepo", repo)

    rc, stdout, stderr = module.run_command(cmd, check_rc=True)
    return rc, stdout, stderr


def get_zypper_version(module):
    rc, stdout, stderr = module.run_command([module.get_bin_path("zypper", required=True), "--version"])
    if rc != 0 or not stdout.startswith("zypper "):
        return LooseVersion("1.0")
    return LooseVersion(stdout.split()[1])


def runrefreshrepo(module, auto_import_keys=False, shortname=None):
    "Forces zypper to refresh repo metadata."
    if auto_import_keys:
        cmd = _get_cmd(module, "--gpg-auto-import-keys", "refresh", "--force")
    else:
        cmd = _get_cmd(module, "refresh", "--force")
    if shortname is not None:
        cmd.extend(["-r", shortname])

    rc, stdout, stderr = module.run_command(cmd, check_rc=True)
    return rc, stdout, stderr


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(),
            repo=dict(),
            state=dict(choices=["present", "absent"], default="present"),
            runrefresh=dict(default=False, type="bool"),
            description=dict(),
            disable_gpg_check=dict(type="bool"),
            autorefresh=dict(type="bool", aliases=["refresh"]),
            priority=dict(type="int"),
            enabled=dict(type="bool"),
            overwrite_multiple=dict(default=False, type="bool"),
            auto_import_keys=dict(default=False, type="bool"),
        ),
        supports_check_mode=False,
        required_one_of=[["state", "runrefresh"]],
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    repo = module.params["repo"]
    alias = module.params["name"]
    state = module.params["state"]
    overwrite_multiple = module.params["overwrite_multiple"]
    auto_import_keys = module.params["auto_import_keys"]
    runrefresh = module.params["runrefresh"]

    zypper_version = get_zypper_version(module)

    repodata = {
        "url": repo,
        "alias": alias,
        "name": module.params["description"],
        "priority": module.params["priority"],
    }
    # rewrite bools in the language that zypper lr -x provides for easier comparison
    # only set if the user explicitly provided the parameter (None means unset)
    if module.params["enabled"] is not None:
        repodata["enabled"] = "1" if module.params["enabled"] else "0"
    if module.params["disable_gpg_check"] is not None:
        repodata["gpgcheck"] = "0" if module.params["disable_gpg_check"] else "1"
    if module.params["autorefresh"] is not None:
        repodata["autorefresh"] = "1" if module.params["autorefresh"] else "0"

    def exit_unchanged():
        module.exit_json(changed=False, repodata=repodata, state=state)

    # Check run-time module parameters
    if repo == "*" or alias == "*":
        if runrefresh:
            runrefreshrepo(module, auto_import_keys)
            module.exit_json(changed=False, runrefresh=True)
        else:
            module.fail_json(msg="repo=* can only be used with the runrefresh option.")

    if state == "present" and not repo:
        module.fail_json(msg="Module option state=present requires repo")
    if state == "absent" and not repo and not alias:
        module.fail_json(msg="Alias or repo parameter required when state=absent")

    if repo and repo.endswith(".repo"):
        if alias:
            module.fail_json(msg="Incompatible option: 'name'. Do not use name when adding .repo files")
    else:
        if not alias and state == "present":
            module.fail_json(msg="Name required when adding non-repo files.")

    # Download / Open and parse .repo file to ensure idempotency
    if repo and repo.endswith(".repo"):
        repofile_text = None
        if repo.startswith(("http://", "https://")):
            response, info = fetch_url(module=module, url=repo, force=True)
            if not response or info["status"] != 200:
                if state == "absent":
                    # Fall back to alias derived from URL filename (convention: section == basename)
                    module.warn(
                        f"Could not download .repo file from '{repo}'; will try to identify repository by alias"
                    )
                    alias = alias or repo.rsplit("/", 1)[-1][:-5]
                    repodata["alias"] = alias
                else:
                    module.fail_json(msg="Error downloading .repo file from provided URL")
            else:
                repofile_text = to_text(response.read(), errors="surrogate_or_strict")
        else:
            try:
                with open(repo, encoding="utf-8") as file:
                    repofile_text = file.read()
            except OSError:
                if state == "absent":
                    module.warn(f"Could not open .repo file at '{repo}'; will try to identify repository by alias")
                    alias = alias or repo.rsplit("/", 1)[-1][:-5]
                    repodata["alias"] = alias
                else:
                    module.fail_json(msg="Error opening .repo file from provided path")

        if repofile_text is not None:
            repofile = configparser.ConfigParser()
            try:
                repofile.read_file(StringIO(repofile_text))
            except configparser.Error:
                module.fail_json(msg="Invalid format, .repo file could not be parsed")

            # No support for .repo file with zero or more than one repository
            if len(repofile.sections()) != 1:
                err = f"Invalid format, .repo file contains {len(repofile.sections())} repositories, expected 1"
                module.fail_json(msg=err)

            section = repofile.sections()[0]
            repofile_items = dict(repofile.items(section))
            # Only proceed if at least baseurl is available
            if "baseurl" not in repofile_items:
                module.fail_json(msg="No baseurl found in .repo file")

            # Set alias (name) and url based on values from .repo file
            alias = section
            repodata["alias"] = section
            repodata["url"] = repofile_items["baseurl"]

            # If gpgkey is part of the .repo file, auto import key
            if "gpgkey" in repofile_items:
                auto_import_keys = True

            # Map additional values, if available
            if "name" in repofile_items:
                repodata["name"] = repofile_items["name"]
            # Use .repo file values only for settings the user did not explicitly provide
            for key in ("enabled", "autorefresh", "gpgcheck"):
                if key not in repodata and key in repofile_items:
                    repodata[key] = repofile_items[key]

    # Apply defaults for any settings not provided by the user or the .repo file
    repodata.setdefault("enabled", "1")
    repodata.setdefault("gpgcheck", "1")
    repodata.setdefault("autorefresh", "1")

    exists, mod, old_repos = repo_exists(module, repodata, overwrite_multiple)

    if alias:
        shortname = alias
    else:
        shortname = repo

    if state == "present":
        if exists and not mod:
            if runrefresh:
                runrefreshrepo(module, auto_import_keys, shortname)
            exit_unchanged()
        rc, stdout, stderr = addmodify_repo(module, repodata, old_repos, zypper_version)
        if rc == 0 and (runrefresh or auto_import_keys):
            runrefreshrepo(module, auto_import_keys, shortname)
    elif state == "absent":
        if not exists:
            exit_unchanged()
        rc, stdout, stderr = remove_repo(module, shortname)

    if rc == 0:
        module.exit_json(changed=True, repodata=repodata, state=state)
    else:
        module.fail_json(
            msg=f"Zypper failed with rc {rc}", rc=rc, stdout=stdout, stderr=stderr, repodata=repodata, state=state
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python


# Copyright (c) 2019, Jon Ellis (@JonEllis) <ellis.jp@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: sudoers
short_description: Manage sudoers files
version_added: "4.3.0"
description:
  - This module allows for the manipulation of sudoers files.
author:
  - "Jon Ellis (@JonEllis) <ellis.jp@gmail.com>"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  commands:
    description:
      - The commands allowed by the sudoers rule.
      - Multiple can be added by passing a list of commands.
      - Use V(ALL) for all commands.
    type: list
    elements: str
  group:
    description:
      - The name of the group for the sudoers rule.
      - This option cannot be used in conjunction with O(user).
    type: str
  name:
    required: true
    description:
      - The name of the sudoers rule.
      - This is used for the filename for the sudoers file managed by this rule.
    type: str
  noexec:
    description:
      - Whether a command is prevented to run further commands itself.
    default: false
    type: bool
    version_added: 8.4.0
  nopassword:
    description:
      - Whether a password is not required when command is run with sudo.
    default: true
    type: bool
  setenv:
    description:
      - Whether to allow keeping the environment when command is run with sudo.
    default: false
    type: bool
    version_added: 6.3.0
  host:
    description:
      - Specify the host the rule is for.
    default: ALL
    type: str
    version_added: 6.2.0
  runas:
    description:
      - Specify the target user the command(s) runs as.
    type: str
    version_added: 4.7.0
  sudoers_path:
    description:
      - The path which sudoers config files are managed in.
    default: /etc/sudoers.d
    type: str
  state:
    default: "present"
    choices:
      - present
      - absent
    description:
      - Whether the rule should exist or not.
    type: str
  user:
    description:
      - The name of the user for the sudoers rule.
      - This option cannot be used in conjunction with O(group).
    type: str
  defaults:
    description:
      - A list of C(Defaults) directives to include in the sudoers rule file.
      - Each entry is written as a C(Defaults) line scoped to the user or group specified in the rule.
      - For example, V(!targetpw) becomes C(Defaults:%group !targetpw) for a group rule
        or C(Defaults:user !targetpw) for a user rule.
      - The directives are placed before the privilege rule in the generated file.
    type: list
    elements: str
    version_added: 13.1.0
  validation:
    description:
      - If V(absent), the sudoers rule is added without validation.
      - If V(detect) and C(visudo) is available, then the sudoers rule is validated by C(visudo).
      - If V(required), C(visudo) must be available to validate the sudoers rule.
    type: str
    default: detect
    choices: [absent, detect, required]
    version_added: 5.2.0
"""

EXAMPLES = r"""
- name: Allow the backup user to sudo /usr/local/bin/backup
  community.general.sudoers:
    name: allow-backup
    state: present
    user: backup
    commands: /usr/local/bin/backup

- name: Allow the bob user to run any commands as alice with sudo -u alice
  community.general.sudoers:
    name: bob-do-as-alice
    state: present
    user: bob
    runas: alice
    commands: ALL

- name: >-
    Allow the monitoring group to run sudo /usr/local/bin/gather-app-metrics
    without requiring a password on the host called webserver
  community.general.sudoers:
    name: monitor-app
    group: monitoring
    host: webserver
    commands: /usr/local/bin/gather-app-metrics

- name: >-
    Allow the alice user to run sudo /bin/systemctl restart my-service or
    sudo /bin/systemctl reload my-service, but a password is required
  community.general.sudoers:
    name: alice-service
    user: alice
    commands:
      - /bin/systemctl restart my-service
      - /bin/systemctl reload my-service
    nopassword: false

- name: Revoke the previous sudo grants given to the alice user
  community.general.sudoers:
    name: alice-service
    state: absent

- name: Allow alice to sudo /usr/local/bin/upload and keep env variables
  community.general.sudoers:
    name: allow-alice-upload
    user: alice
    commands: /usr/local/bin/upload
    setenv: true

- name: >-
    Allow alice to sudo /usr/bin/less but prevent less from
    running further commands itself
  community.general.sudoers:
    name: allow-alice-restricted-less
    user: alice
    commands: /usr/bin/less
    noexec: true

- name: Allow members of the operators group to sudo with their user password, overriding targetpw default
  community.general.sudoers:
    name: operators
    group: operators
    commands: ALL
    nopassword: false
    defaults:
      - "!targetpw"
"""

import os

from ansible.module_utils.basic import AnsibleModule


class Sudoers:
    FILE_MODE = 0o440

    def __init__(self, module):
        self.module = module

        self.check_mode = module.check_mode
        self.name = module.params["name"]
        self.user = module.params["user"]
        self.group = module.params["group"]
        self.state = module.params["state"]
        self.noexec = module.params["noexec"]
        self.nopassword = module.params["nopassword"]
        self.setenv = module.params["setenv"]
        self.host = module.params["host"]
        self.runas = module.params["runas"]
        self.sudoers_path = module.params["sudoers_path"]
        self.file = os.path.join(self.sudoers_path, self.name)
        self.commands = module.params["commands"]
        self.defaults = module.params["defaults"]
        self.validation = module.params["validation"]

    def write(self):
        if self.check_mode:
            return

        with open(self.file, "w") as f:
            f.write(self.content())

        os.chmod(self.file, self.FILE_MODE)

    def delete(self):
        if self.check_mode:
            return

        os.remove(self.file)

    def exists(self):
        return os.path.exists(self.file)

    def matches(self):
        with open(self.file) as f:
            content_matches = f.read() == self.content()

        current_mode = os.stat(self.file).st_mode & 0o777
        mode_matches = current_mode == self.FILE_MODE

        return content_matches and mode_matches

    def content(self):
        if self.user:
            owner = self.user
        elif self.group:
            owner = f"%{self.group}"

        if self.defaults:
            defaults_lines = [f"Defaults:{owner} {d}" for d in self.defaults]
            defaults_str = "\n".join(defaults_lines) + "\n"
        else:
            defaults_str = ""

        commands_str = ", ".join(self.commands)
        noexec_str = "NOEXEC:" if self.noexec else ""
        nopasswd_str = "NOPASSWD:" if self.nopassword else ""
        setenv_str = "SETENV:" if self.setenv else ""
        runas_str = f"({self.runas})" if self.runas is not None else ""
        return f"{defaults_str}{owner} {self.host}={runas_str}{noexec_str}{nopasswd_str}{setenv_str} {commands_str}\n"

    def validate(self):
        if self.validation == "absent":
            return

        visudo_path = self.module.get_bin_path("visudo", required=self.validation == "required")
        if visudo_path is None:
            return

        check_command = [visudo_path, "-c", "-f", "-"]
        rc, stdout, stderr = self.module.run_command(check_command, data=self.content())

        if rc != 0:
            self.module.fail_json(
                msg=f"Failed to validate sudoers rule:\n{stdout or stderr}", stdout=stdout, stderr=stderr
            )

    def run(self):
        if self.state == "absent":
            if self.exists():
                self.delete()
                return True
            else:
                return False

        self.validate()

        if self.exists() and self.matches():
            return False

        self.write()
        return True


def main():
    argument_spec = {
        "commands": {
            "type": "list",
            "elements": "str",
        },
        "defaults": {
            "type": "list",
            "elements": "str",
        },
        "group": {},
        "name": {
            "required": True,
        },
        "noexec": {
            "type": "bool",
            "default": False,
        },
        "nopassword": {
            "type": "bool",
            "default": True,
        },
        "setenv": {
            "type": "bool",
            "default": False,
        },
        "host": {
            "type": "str",
            "default": "ALL",
        },
        "runas": {
            "type": "str",
            "default": None,
        },
        "sudoers_path": {
            "type": "str",
            "default": "/etc/sudoers.d",
        },
        "state": {
            "default": "present",
            "choices": ["present", "absent"],
        },
        "user": {},
        "validation": {"default": "detect", "choices": ["absent", "detect", "required"]},
    }

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["user", "group"]],
        supports_check_mode=True,
        required_if=[("state", "present", ["commands"])],
    )

    sudoers = Sudoers(module)

    try:
        changed = sudoers.run()
        module.exit_json(changed=changed)
    except Exception as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_networks_moves
short_description: Resource module for organizations _networks _moves
description:
  - Manage operation create of the resource organizations _networks _moves. - > Move
    networks from one organization to another. For more information and caveats around
    network move limitations see the Network Portability documentation.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  network:
    description: Network to be moved.
    suboptions:
      id:
        description: ID of the network to move.
        type: str
    type: dict
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  organizations:
    description: Organizations involved in the network move.
    suboptions:
      target:
        description: The organization to which the network is moving.
        suboptions:
          id:
            description: ID of the organization to which the network is moving.
            type: str
        type: dict
    type: dict
  simulate:
    description: If true, simulates the network move and validates the operation without
      committing changes. The network will remain in the source organization.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createNetworkMove
    description: Complete reference of the createNetworkMove API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-move
notes:
  - SDK Method used are
    organizations.Organizations.create_network_move,
  - Paths used are
    post /organizations/{organizationId}/networks/moves,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_networks_moves:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    network:
      id: N_569142402909112097
    organizationId: string
    organizations:
      target:
        id: '146308'
    simulate: false
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "moveId": "string",
      "initiator": {
        "admin": {
          "id": "string"
        }
      },
      "organizations": {
        "source": {
          "id": "string"
        },
        "target": {
          "id": "string"
        }
      },
      "network": {
        "id": "string"
      },
      "createdAt": "string",
      "lastUpdatedAt": "string",
      "result": {
        "status": "string",
        "reason": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_static_routes
short_description: Resource module for networks _appliance _static _routes
description:
  - Manage operations create, update and delete of the resource networks _appliance
    _static _routes.
  - Add a static route for an MX or teleworker network.
  - Delete a static route from an MX or teleworker network.
  - Update a static route for an MX or teleworker network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  enabled:
    description: Whether the route should be enabled or not.
    type: bool
  fixedIpAssignments:
    description: Fixed DHCP IP assignments on the route.
    type: dict
  gatewayIp:
    description: Gateway IP address (next hop).
    type: str
  gatewayVlanId:
    description: Gateway VLAN ID.
    type: str
  name:
    description: Name of the route.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  reservedIpRanges:
    description: DHCP reserved IP ranges.
    elements: dict
    suboptions:
      comment:
        description: Description of the range.
        type: str
      end:
        description: Last address in the reserved range.
        type: str
      start:
        description: First address in the reserved range.
        type: str
    type: list
  staticRouteId:
    description: StaticRouteId path parameter. Static route ID.
    type: str
  subnet:
    description: Subnet of the route.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance createNetworkApplianceStaticRoute
    description: Complete reference of the createNetworkApplianceStaticRoute API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-appliance-static-route
  - name: Cisco Meraki documentation for appliance deleteNetworkApplianceStaticRoute
    description: Complete reference of the deleteNetworkApplianceStaticRoute API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-appliance-static-route
  - name: Cisco Meraki documentation for appliance updateNetworkApplianceStaticRoute
    description: Complete reference of the updateNetworkApplianceStaticRoute API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-static-route
notes:
  - SDK Method used are
    appliance.Appliance.create_network_appliance_static_route,
    appliance.Appliance.delete_network_appliance_static_route,
    appliance.Appliance.update_network_appliance_static_route,
  - Paths used are
    post /networks/{networkId}/appliance/staticRoutes,
    delete /networks/{networkId}/appliance/staticRoutes/{staticRouteId},
    put /networks/{networkId}/appliance/staticRoutes/{staticRouteId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_appliance_static_routes:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    gatewayIp: 1.2.3.5
    gatewayVlanId: '100'
    name: My route
    networkId: string
    subnet: 192.168.1.0/24
- name: Update by id
  cisco.meraki.networks_appliance_static_routes:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    enabled: true
    fixedIpAssignments:
      22:33:44:55:66:77:
        ip: 1.2.3.4
        name: Some client name
    gatewayIp: 1.2.3.5
    gatewayVlanId: '100'
    name: My route
    networkId: string
    reservedIpRanges:
      - comment: A reserved IP range
        end: 192.168.1.1
        start: 192.168.1.0
    staticRouteId: string
    subnet: 192.168.1.0/24
- name: Delete by id
  cisco.meraki.networks_appliance_static_routes:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    staticRouteId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "ipVersion": 0,
      "networkId": "string",
      "enabled": true,
      "name": "string",
      "subnet": "string",
      "gatewayIp": "string",
      "fixedIpAssignments": {},
      "reservedIpRanges": [
        {
          "start": "string",
          "end": "string",
          "comment": "string"
        }
      ],
      "gatewayVlanId": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_server_pool_qualification_policy
short_description: Server Pool Qualification Policy configuration for Cisco Intersight
description:
  - Manages Server Pool Qualification Policy configuration on Cisco Intersight.
  - A Server Pool Qualification Policy defines conditions to qualify servers for resource pools based on various hardware and configuration attributes.
  - Supports multiple qualifier types including Domain, Rack Server, Blade, Tag, Memory, GPU, Processor, and Network Adaptor qualifiers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/resourcepool/QualificationPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Server Pool Qualification Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Server Pool Qualification Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  domain_qualifier:
    description:
      - Domain qualifier for Fabric Interconnect and domain-based qualification.
    type: dict
    suboptions:
      fabric_interconnect_pids:
        description:
          - List of Fabric Interconnect PIDs to qualify resources.
          - Qualifies resources based on the PID property of Fabric Interconnects.
        type: list
        elements: str
      domain_names:
        description:
          - List of domain names to qualify resources.
          - Qualifies resources based on the DomainName property.
        type: list
        elements: str
  rack_server_qualifier:
    description:
      - Rack server qualifier for qualifying rack-based servers.
    type: dict
    suboptions:
      rack_id_ranges:
        description:
          - List of rack ID ranges to qualify servers.
          - Each range specifies min_value and max_value for rack IDs.
        type: list
        elements: dict
        suboptions:
          min_value:
            description:
              - Minimum rack ID value (1-256).
            type: int
            required: true
          max_value:
            description:
              - Maximum rack ID value (min_value-256).
            type: int
            required: true
      pids:
        description:
          - List of rack server PIDs to qualify (maximum 20).
          - Qualifies rack servers based on their PID.
        type: list
        elements: str
      asset_tags:
        description:
          - List of asset tags to qualify servers.
          - Qualifies servers based on the AssetTag property.
        type: list
        elements: str
      user_labels:
        description:
          - List of user labels to qualify servers.
          - Qualifies servers based on the UserLabel property.
        type: list
        elements: str
  blade_qualifier:
    description:
      - Blade qualifier for qualifying blade-based servers.
    type: dict
    suboptions:
      pids:
        description:
          - List of blade server PIDs to qualify (maximum 20).
          - Qualifies blade servers based on their PID.
        type: list
        elements: str
      chassis_pids:
        description:
          - List of chassis PIDs to qualify (maximum 20).
          - Qualifies blade servers based on chassis PID.
        type: list
        elements: str
      asset_tags:
        description:
          - List of asset tags to qualify blade servers.
          - Qualifies servers based on the AssetTag property.
        type: list
        elements: str
      user_labels:
        description:
          - List of user labels to qualify blade servers.
          - Qualifies servers based on the UserLabel property.
        type: list
        elements: str
      chassis_slot_ranges:
        description:
          - List of chassis and slot ID ranges to qualify blade servers.
        type: list
        elements: dict
        suboptions:
          chassis_id_range:
            description:
              - Chassis ID range specification.
            type: dict
            required: true
            suboptions:
              min_value:
                description:
                  - Minimum chassis ID value (1-40).
                type: int
                required: true
              max_value:
                description:
                  - Maximum chassis ID value (min_value-40).
                type: int
                required: true
          slot_id_ranges:
            description:
              - List of slot ID ranges within the chassis.
              - Can only be specified if chassis_id_range is provided.
            type: list
            elements: dict
            suboptions:
              min_value:
                description:
                  - Minimum slot ID value (1-8).
                type: int
                required: true
              max_value:
                description:
                  - Maximum slot ID value (min_value-8).
                type: int
                required: true
  tag_qualifier:
    description:
      - Tag qualifier for qualifying resources based on tags.
    type: dict
    suboptions:
      server_tags:
        description:
          - List of server tags in key-value format.
        type: list
        elements: dict
        suboptions:
          key:
            description:
              - Tag key.
            type: str
            required: true
          value:
            description:
              - Tag value.
            type: str
            required: true
      domain_profile_tags:
        description:
          - List of domain profile tags in key-value format.
        type: list
        elements: dict
        suboptions:
          key:
            description:
              - Tag key.
            type: str
            required: true
          value:
            description:
              - Tag value.
            type: str
            required: true
      chassis_tags:
        description:
          - List of chassis tags in key-value format.
        type: list
        elements: dict
        suboptions:
          key:
            description:
              - Tag key.
            type: str
            required: true
          value:
            description:
              - Tag value.
            type: str
            required: true
  memory_qualifier:
    description:
      - Memory qualifier for qualifying servers based on memory specifications.
    type: dict
    suboptions:
      capacity_range:
        description:
          - Memory capacity range in GiB.
        type: dict
        suboptions:
          min_value:
            description:
              - Minimum memory capacity in GiB (1-999999).
            type: int
            required: true
          max_value:
            description:
              - Maximum memory capacity in GiB (min_value-999999).
            type: int
            required: true
      units_range:
        description:
          - Number of memory units range.
        type: dict
        suboptions:
          min_value:
            description:
              - Minimum number of memory units (1-99999).
            type: int
            required: true
          max_value:
            description:
              - Maximum number of memory units (min_value-99999).
            type: int
            required: true
  gpu_qualifier:
    description:
      - GPU qualifier for qualifying servers based on GPU specifications.
      - If not specified, a default GPU qualifier with C(servers_without_gpu) evaluation type is automatically added.
    type: dict
    suboptions:
      evaluation_type:
        description:
          - GPU evaluation type for server qualification.
          - C(servers_without_gpu) - Qualifies servers without GPUs (default).
          - C(all_servers) - Qualifies all servers regardless of GPU presence.
          - C(servers_with_gpu) - Qualifies only servers with GPUs.
        type: str
        choices: [servers_without_gpu, all_servers, servers_with_gpu]
        default: servers_without_gpu
      gpu_count_range:
        description:
          - GPU count range (only applicable when evaluation_type is servers_with_gpu).
        type: dict
        suboptions:
          min_value:
            description:
              - Minimum number of GPUs (1-16).
            type: int
            required: true
          max_value:
            description:
              - Maximum number of GPUs (min_value-16).
            type: int
            required: true
      pids:
        description:
          - List of GPU PIDs to qualify (maximum 20, only for servers_with_gpu).
        type: list
        elements: str
      vendor:
        description:
          - GPU vendor (only applicable when evaluation_type is servers_with_gpu).
        type: str
        choices: [nvidia, intel, amd]
  processor_qualifier:
    description:
      - Processor qualifier for qualifying servers based on CPU specifications.
    type: dict
    suboptions:
      cores_range:
        description:
          - CPU cores range.
        type: dict
        suboptions:
          min_value:
            description:
              - Minimum number of CPU cores (1-9999).
            type: int
            required: true
          max_value:
            description:
              - Maximum number of CPU cores (min_value-9999).
            type: int
            required: true
      speed_range:
        description:
          - CPU speed range in GHz.
        type: dict
        suboptions:
          min_value:
            description:
              - Minimum CPU speed in GHz (1-99).
            type: int
            required: true
          max_value:
            description:
              - Maximum CPU speed in GHz (min_value-99).
            type: int
            required: true
      pids:
        description:
          - List of processor PIDs to qualify (maximum 100).
        type: list
        elements: str
      vendor:
        description:
          - Processor vendor.
        type: str
        choices: [intel, amd]
  network_adaptor_qualifier:
    description:
      - Network adaptor qualifier for qualifying servers based on network adaptor count.
    type: dict
    suboptions:
      adaptors_range:
        description:
          - Number of network adaptors range.
        type: dict
        suboptions:
          min_value:
            description:
              - Minimum number of network adaptors (1-16).
            type: int
            required: true
          max_value:
            description:
              - Maximum number of network adaptors (min_value-16).
            type: int
            required: true
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create minimal Server Pool Qualification Policy
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "ServerPool-Minimal-Policy"
    description: "Minimal policy - GPU qualifier automatically added with servers_without_gpu"
    state: present

- name: Create Server Pool Qualification Policy with domain and rack qualifiers
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "ServerPool-Qual-Policy-01"
    description: "Server pool qualification with domain and rack filters"
    domain_qualifier:
      fabric_interconnect_pids:
        - ucs-fi-6454
        - ucs-fi-64108
      domain_names:
        - "AC08-6454"
    rack_server_qualifier:
      rack_id_ranges:
        - min_value: 2
          max_value: 4
        - min_value: 5
          max_value: 8
      pids:
        - "UCSC-C245-M8SX"
        - "UCSC-C220-M8S"
      asset_tags:
        - "production"
      user_labels:
        - "datacenter-a"
    state: present

- name: Create Server Pool Qualification Policy with blade and memory qualifiers
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ServerPool-Blade-Policy"
    blade_qualifier:
      pids:
        - "UCSB-B480-M5"
        - "UCSB-B200-M5"
      chassis_pids:
        - "N20-C6508"
      chassis_slot_ranges:
        - chassis_id_range:
            min_value: 2
            max_value: 10
          slot_id_ranges:
            - min_value: 4
              max_value: 6
    memory_qualifier:
      capacity_range:
        min_value: 64
        max_value: 512
      units_range:
        min_value: 10
        max_value: 20
    state: present

- name: Create Server Pool Qualification Policy with GPU requirements
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ServerPool-GPU-Policy"
    description: "Policy for servers with NVIDIA GPUs"
    gpu_qualifier:
      evaluation_type: servers_with_gpu
      gpu_count_range:
        min_value: 2
        max_value: 4
      vendor: nvidia
    processor_qualifier:
      cores_range:
        min_value: 16
        max_value: 64
      vendor: intel
    state: present

- name: Create Server Pool Qualification Policy with tag qualifiers
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ServerPool-Tag-Policy"
    tag_qualifier:
      server_tags:
        - key: "environment"
          value: "production"
        - key: "tier"
          value: "1"
      chassis_tags:
        - key: "location"
          value: "datacenter-a"
    network_adaptor_qualifier:
      adaptors_range:
        min_value: 2
        max_value: 8
    state: present

- name: Create comprehensive Server Pool Qualification Policy
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Production"
    name: "ServerPool-Comprehensive-Policy"
    description: "Comprehensive qualification policy with multiple qualifiers"
    domain_qualifier:
      fabric_interconnect_pids:
        - ucs-fi-6454
    rack_server_qualifier:
      pids:
        - "UCSC-C220-M6S"
        - "UCSC-C240-M6S"
    blade_qualifier:
      pids:
        - "UCSX-210C-M7"
        - "UCSX-410C-M7"
    memory_qualifier:
      capacity_range:
        min_value: 128
        max_value: 1024
    processor_qualifier:
      cores_range:
        min_value: 32
        max_value: 128
      vendor: intel
    gpu_qualifier:
      evaluation_type: servers_without_gpu
    network_adaptor_qualifier:
      adaptors_range:
        min_value: 2
        max_value: 4
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Delete Server Pool Qualification Policy
  cisco.intersight.intersight_server_pool_qualification_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ServerPool-Qual-Policy-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "ServerPool-Qual-Policy-01",
        "ObjectType": "resourcepool.QualificationPolicy",
        "Organization": {
            "Moid": "675450ee69726530014753e2",
            "ObjectType": "organization.Organization"
        },
        "Qualifiers": [
            {
                "FabricInterConnectPids": ["UCS-FI-6454", "UCS-FI-64108"],
                "DomainNames": ["AC08-6454"],
                "ObjectType": "resource.DomainQualifier"
            },
            {
                "RackIdRange": [{"MinValue": 2, "MaxValue": 4}],
                "Pids": ["UCSC-C245-M8SX"],
                "ObjectType": "resource.RackServerQualifier"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def build_domain_qualifier(domain_qualifier):
    """Build domain qualifier API body."""
    qualifier = {'ObjectType': 'resource.DomainQualifier'}

    if domain_qualifier.get('fabric_interconnect_pids'):
        # Convert lowercase choices to API format (uppercase with hyphens)
        qualifier['FabricInterConnectPids'] = [
            pid.upper() for pid in domain_qualifier['fabric_interconnect_pids']
        ]

    if domain_qualifier.get('domain_names'):
        qualifier['DomainNames'] = domain_qualifier['domain_names']

    return qualifier


def build_rack_server_qualifier(rack_server_qualifier):
    """Build rack server qualifier API body."""
    qualifier = {'ObjectType': 'resource.RackServerQualifier'}

    if rack_server_qualifier.get('rack_id_ranges'):
        qualifier['RackIdRange'] = [
            {'MinValue': range_item['min_value'], 'MaxValue': range_item['max_value']}
            for range_item in rack_server_qualifier['rack_id_ranges']
        ]

    if rack_server_qualifier.get('pids'):
        qualifier['Pids'] = rack_server_qualifier['pids']

    if rack_server_qualifier.get('asset_tags'):
        qualifier['AssetTags'] = rack_server_qualifier['asset_tags']

    if rack_server_qualifier.get('user_labels'):
        qualifier['UserLabels'] = rack_server_qualifier['user_labels']

    return qualifier


def build_blade_qualifier(blade_qualifier):
    """Build blade qualifier API body."""
    qualifier = {'ObjectType': 'resource.BladeQualifier'}

    if blade_qualifier.get('pids'):
        qualifier['Pids'] = blade_qualifier['pids']

    if blade_qualifier.get('chassis_pids'):
        qualifier['ChassisPids'] = blade_qualifier['chassis_pids']

    if blade_qualifier.get('asset_tags'):
        qualifier['AssetTags'] = blade_qualifier['asset_tags']

    if blade_qualifier.get('user_labels'):
        qualifier['UserLabels'] = blade_qualifier['user_labels']

    if blade_qualifier.get('chassis_slot_ranges'):
        chassis_slot_list = []
        for range_item in blade_qualifier['chassis_slot_ranges']:
            chassis_slot = {
                'ChassisIdRange': {
                    'MinValue': range_item['chassis_id_range']['min_value'],
                    'MaxValue': range_item['chassis_id_range']['max_value']
                }
            }
            if range_item.get('slot_id_ranges'):
                chassis_slot['SlotIdRanges'] = [
                    {'MinValue': slot['min_value'], 'MaxValue': slot['max_value']}
                    for slot in range_item['slot_id_ranges']
                ]
            chassis_slot_list.append(chassis_slot)
        qualifier['ChassisAndSlotIdRange'] = chassis_slot_list

    return qualifier


def build_tag_qualifier(tag_qualifier):
    """Build tag qualifier API body."""
    qualifier = {'ObjectType': 'resource.TagQualifier'}

    if tag_qualifier.get('server_tags'):
        qualifier['ServerTags'] = [
            {'Key': tag['key'], 'Value': tag['value']}
            for tag in tag_qualifier['server_tags']
        ]

    if tag_qualifier.get('domain_profile_tags'):
        qualifier['DomainProfileTags'] = [
            {'Key': tag['key'], 'Value': tag['value']}
            for tag in tag_qualifier['domain_profile_tags']
        ]

    if tag_qualifier.get('chassis_tags'):
        qualifier['ChassisTags'] = [
            {'Key': tag['key'], 'Value': tag['value']}
            for tag in tag_qualifier['chassis_tags']
        ]

    return qualifier


def build_memory_qualifier(memory_qualifier):
    """Build memory qualifier API body."""
    qualifier = {'ObjectType': 'resource.MemoryQualifier'}

    if memory_qualifier.get('capacity_range'):
        qualifier['MemoryCapacityRange'] = {
            'MinValue': memory_qualifier['capacity_range']['min_value'],
            'MaxValue': memory_qualifier['capacity_range']['max_value']
        }

    if memory_qualifier.get('units_range'):
        qualifier['MemoryUnitsRange'] = {
            'MinValue': memory_qualifier['units_range']['min_value'],
            'MaxValue': memory_qualifier['units_range']['max_value']
        }

    return qualifier


def build_gpu_qualifier(gpu_qualifier):
    """Build GPU qualifier API body."""
    qualifier = {'ObjectType': 'resource.GpuQualifier'}

    # Convert evaluation type to API format
    eval_type_map = {
        'servers_without_gpu': 'ServerWithoutGpu',
        'all_servers': 'Unspecified',
        'servers_with_gpu': 'ServerWithGpu'
    }
    eval_type = gpu_qualifier.get('evaluation_type', 'servers_without_gpu')
    qualifier['GpuEvaluationType'] = eval_type_map[eval_type]

    # GPU-specific fields only apply for ServerWithGpu
    if eval_type == 'servers_with_gpu':
        if gpu_qualifier.get('gpu_count_range'):
            qualifier['GpuCountRange'] = {
                'MinValue': gpu_qualifier['gpu_count_range']['min_value'],
                'MaxValue': gpu_qualifier['gpu_count_range']['max_value']
            }

        if gpu_qualifier.get('pids'):
            qualifier['Pids'] = gpu_qualifier['pids']

        if gpu_qualifier.get('vendor'):
            # Convert vendor to API format
            vendor_map = {
                'nvidia': 'NVIDIA',
                'intel': 'Intel',
                'amd': 'AMD'
            }
            qualifier['Vendor'] = vendor_map[gpu_qualifier['vendor']]

    return qualifier


def build_processor_qualifier(processor_qualifier):
    """Build processor qualifier API body."""
    qualifier = {'ObjectType': 'resource.ProcessorQualifier'}

    if processor_qualifier.get('cores_range'):
        qualifier['CpuCoresRange'] = {
            'MinValue': processor_qualifier['cores_range']['min_value'],
            'MaxValue': processor_qualifier['cores_range']['max_value']
        }

    if processor_qualifier.get('speed_range'):
        qualifier['SpeedRange'] = {
            'MinValue': processor_qualifier['speed_range']['min_value'],
            'MaxValue': processor_qualifier['speed_range']['max_value']
        }

    if processor_qualifier.get('pids'):
        qualifier['Pids'] = processor_qualifier['pids']

    if processor_qualifier.get('vendor'):
        # Convert vendor to API format
        vendor_map = {
            'intel': 'Intel(R) Corporation',
            'amd': 'AMD'
        }
        qualifier['Vendor'] = vendor_map[processor_qualifier['vendor']]

    return qualifier


def build_network_adaptor_qualifier(network_adaptor_qualifier):
    """Build network adaptor qualifier API body."""
    qualifier = {'ObjectType': 'resource.NetworkAdaptorQualifier'}

    if network_adaptor_qualifier.get('adaptors_range'):
        qualifier['AdaptorsRange'] = {
            'MinValue': network_adaptor_qualifier['adaptors_range']['min_value'],
            'MaxValue': network_adaptor_qualifier['adaptors_range']['max_value']
        }

    return qualifier


def validate_range_values(module, range_dict, range_name, min_limit, max_limit):
    """Validate that range min/max values are within limits and min <= max."""
    if not range_dict:
        return

    min_val = range_dict.get('min_value')
    max_val = range_dict.get('max_value')

    if min_val is not None and (min_val < min_limit or min_val > max_limit):
        module.fail_json(msg=f"{range_name} min_value must be between {min_limit} and {max_limit}")

    if max_val is not None and (max_val < min_limit or max_val > max_limit):
        module.fail_json(msg=f"{range_name} max_value must be between {min_limit} and {max_limit}")

    if min_val is not None and max_val is not None and min_val > max_val:
        module.fail_json(msg=f"{range_name} min_value cannot be greater than max_value")


def validate_list_length(module, list_param, max_length, param_name):
    """Validate that a list parameter does not exceed maximum length."""
    if list_param and len(list_param) > max_length:
        module.fail_json(msg=f"{param_name} cannot contain more than {max_length} items")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        domain_qualifier=dict(
            type='dict',
            options=dict(
                fabric_interconnect_pids=dict(
                    type='list',
                    elements='str'
                ),
                domain_names=dict(type='list', elements='str')
            )
        ),
        rack_server_qualifier=dict(
            type='dict',
            options=dict(
                rack_id_ranges=dict(
                    type='list',
                    elements='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                ),
                pids=dict(type='list', elements='str'),
                asset_tags=dict(type='list', elements='str'),
                user_labels=dict(type='list', elements='str')
            )
        ),
        blade_qualifier=dict(
            type='dict',
            options=dict(
                pids=dict(type='list', elements='str'),
                chassis_pids=dict(type='list', elements='str'),
                asset_tags=dict(type='list', elements='str'),
                user_labels=dict(type='list', elements='str'),
                chassis_slot_ranges=dict(
                    type='list',
                    elements='dict',
                    options=dict(
                        chassis_id_range=dict(
                            type='dict',
                            required=True,
                            options=dict(
                                min_value=dict(type='int', required=True),
                                max_value=dict(type='int', required=True)
                            )
                        ),
                        slot_id_ranges=dict(
                            type='list',
                            elements='dict',
                            options=dict(
                                min_value=dict(type='int', required=True),
                                max_value=dict(type='int', required=True)
                            )
                        )
                    )
                )
            )
        ),
        tag_qualifier=dict(
            type='dict',
            options=dict(
                server_tags=dict(
                    type='list',
                    elements='dict',
                    options=dict(
                        key=dict(type='str', required=True, no_log=False),
                        value=dict(type='str', required=True)
                    )
                ),
                domain_profile_tags=dict(
                    type='list',
                    elements='dict',
                    options=dict(
                        key=dict(type='str', required=True, no_log=False),
                        value=dict(type='str', required=True)
                    )
                ),
                chassis_tags=dict(
                    type='list',
                    elements='dict',
                    options=dict(
                        key=dict(type='str', required=True, no_log=False),
                        value=dict(type='str', required=True)
                    )
                )
            )
        ),
        memory_qualifier=dict(
            type='dict',
            options=dict(
                capacity_range=dict(
                    type='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                ),
                units_range=dict(
                    type='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                )
            )
        ),
        gpu_qualifier=dict(
            type='dict',
            options=dict(
                evaluation_type=dict(
                    type='str',
                    choices=['servers_without_gpu', 'all_servers', 'servers_with_gpu'],
                    default='servers_without_gpu'
                ),
                gpu_count_range=dict(
                    type='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                ),
                pids=dict(type='list', elements='str'),
                vendor=dict(type='str', choices=['nvidia', 'intel', 'amd'])
            )
        ),
        processor_qualifier=dict(
            type='dict',
            options=dict(
                cores_range=dict(
                    type='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                ),
                speed_range=dict(
                    type='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                ),
                pids=dict(type='list', elements='str'),
                vendor=dict(type='str', choices=['intel', 'amd'])
            )
        ),
        network_adaptor_qualifier=dict(
            type='dict',
            options=dict(
                adaptors_range=dict(
                    type='dict',
                    options=dict(
                        min_value=dict(type='int', required=True),
                        max_value=dict(type='int', required=True)
                    )
                )
            )
        )
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/resourcepool/QualificationPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Build qualifiers list
        qualifiers = []

        # Domain qualifier
        if module.params.get('domain_qualifier'):
            qualifiers.append(build_domain_qualifier(module.params['domain_qualifier']))

        # Rack server qualifier
        if module.params.get('rack_server_qualifier'):
            rack_qual = module.params['rack_server_qualifier']

            # Validate rack ID ranges
            if rack_qual.get('rack_id_ranges'):
                for rack_range in rack_qual['rack_id_ranges']:
                    validate_range_values(module, rack_range, 'Rack ID range', 1, 256)

            # Validate PID list length
            validate_list_length(module, rack_qual.get('pids'), 20, 'Rack server PIDs')

            qualifiers.append(build_rack_server_qualifier(rack_qual))

        # Blade qualifier
        if module.params.get('blade_qualifier'):
            blade_qual = module.params['blade_qualifier']

            # Validate PID list lengths
            validate_list_length(module, blade_qual.get('pids'), 20, 'Blade server PIDs')
            validate_list_length(module, blade_qual.get('chassis_pids'), 20, 'Chassis PIDs')

            # Validate chassis and slot ranges
            if blade_qual.get('chassis_slot_ranges'):
                for range_item in blade_qual['chassis_slot_ranges']:
                    validate_range_values(module, range_item['chassis_id_range'],
                                          'Chassis ID range', 1, 40)
                    if range_item.get('slot_id_ranges'):
                        for slot_range in range_item['slot_id_ranges']:
                            validate_range_values(module, slot_range, 'Slot ID range', 1, 8)

            qualifiers.append(build_blade_qualifier(blade_qual))

        # Tag qualifier
        if module.params.get('tag_qualifier'):
            qualifiers.append(build_tag_qualifier(module.params['tag_qualifier']))

        # Memory qualifier
        if module.params.get('memory_qualifier'):
            mem_qual = module.params['memory_qualifier']

            # Validate memory ranges
            validate_range_values(module, mem_qual.get('capacity_range'),
                                  'Memory capacity range', 1, 999999)
            validate_range_values(module, mem_qual.get('units_range'),
                                  'Memory units range', 1, 99999)

            qualifiers.append(build_memory_qualifier(mem_qual))

        # GPU qualifier
        if module.params.get('gpu_qualifier'):
            gpu_qual = module.params['gpu_qualifier']

            # Validate GPU count range
            validate_range_values(module, gpu_qual.get('gpu_count_range'),
                                  'GPU count range', 1, 16)

            # Validate GPU PIDs list length
            validate_list_length(module, gpu_qual.get('pids'), 20, 'GPU PIDs')

            # Validate GPU-specific fields only for servers_with_gpu
            if gpu_qual.get('evaluation_type') != 'servers_with_gpu':
                if gpu_qual.get('gpu_count_range') or gpu_qual.get('pids') or gpu_qual.get('vendor'):
                    module.warn('GPU count, PIDs, and vendor are only applicable when evaluation_type is servers_with_gpu')

            qualifiers.append(build_gpu_qualifier(gpu_qual))
        else:
            # Add default GPU qualifier if not specified
            default_gpu_qual = {'evaluation_type': 'servers_without_gpu'}
            qualifiers.append(build_gpu_qualifier(default_gpu_qual))

        # Processor qualifier
        if module.params.get('processor_qualifier'):
            proc_qual = module.params['processor_qualifier']

            # Validate processor ranges
            validate_range_values(module, proc_qual.get('cores_range'),
                                  'CPU cores range', 1, 9999)
            validate_range_values(module, proc_qual.get('speed_range'),
                                  'CPU speed range', 1, 99)

            # Validate processor PIDs list length
            validate_list_length(module, proc_qual.get('pids'), 100, 'Processor PIDs')

            qualifiers.append(build_processor_qualifier(proc_qual))

        # Network adaptor qualifier
        if module.params.get('network_adaptor_qualifier'):
            net_qual = module.params['network_adaptor_qualifier']

            # Validate adaptors range
            validate_range_values(module, net_qual.get('adaptors_range'),
                                  'Network adaptors range', 1, 16)

            qualifiers.append(build_network_adaptor_qualifier(net_qual))

        # Add qualifiers to API body (always present, at minimum GPU qualifier)
        intersight.api_body['Qualifiers'] = qualifiers

    # Configure the policy
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_devices_radsec_certificates_authorities
short_description: Resource module for organizations _wireless _devices _radsec _certificates
  _authorities
description:
  - Manage operations create and update of the resource organizations _wireless _devices
    _radsec _certificates _authorities. - > Create an organization's RADSEC device
    Certificate Authority CA. Call this endpoint when turning on RADSEC in the firmware
    for the first time. Calling this endpoint starts an asynchronous process to generate
    the CA; call GET afterwards to retrieve the contents of the CA. Note this CA is
    generated and controlled by Meraki. Subsequent calls will not generate a new CA.
    - > Update an organization's RADSEC device Certificate Authority CA state. Note
    this CA is generated and controlled by Meraki. Call this endpoint to update the
    state to "trusted", at which point Meraki will generate device certificates. "trusted"
    means the CA is placed on your RADSEC servers and devices establishing a secure
    connection using certs signed by this CA will pass verification.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  certificateAuthorityId:
    description: The ID of the Certificate Authority to update.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  status:
    description: The "status" to update the Certificate Authority to. Only valid option
      is "trusted".
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless createOrganizationWirelessDevicesRadsecCertificatesAuthority
    description: Complete reference of the createOrganizationWirelessDevicesRadsecCertificatesAuthority
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-wireless-devices-radsec-certificates-authority
  - name: Cisco Meraki documentation for wireless updateOrganizationWirelessDevicesRadsecCertificatesAuthorities
    description: Complete reference of the updateOrganizationWirelessDevicesRadsecCertificatesAuthorities
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-wireless-devices-radsec-certificates-authorities
notes:
  - SDK Method used are
    wireless.Wireless.create_organization_wireless_devices_radsec_certificates_authority,
    wireless.Wireless.update_organization_wireless_devices_radsec_certificates_authorities,
  - Paths used are
    post /organizations/{organizationId}/wireless/devices/radsec/certificates/authorities,
    put /organizations/{organizationId}/wireless/devices/radsec/certificates/authorities,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.organizations_wireless_devices_radsec_certificates_authorities:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    certificateAuthorityId: '1234'
    organizationId: string
    status: trusted
- name: Create
  cisco.meraki.organizations_wireless_devices_radsec_certificates_authorities:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "certificateAuthorityId": "string",
      "status": "string",
      "contents": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_fibre_channel_adapter_policy
short_description: Manage Fibre Channel Adapter Policies for Cisco Intersight
description:
  - Create, update, and delete Fibre Channel Adapter Policies on Cisco Intersight.
  - Fibre Channel Adapter policies configure FC adapter settings for Fibre Channel virtual interfaces.
  - These policies control error recovery, timeouts, retry counts, interrupt modes, queue settings, and LUN configurations.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/FcAdapterPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Fibre Channel Adapter Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Fibre Channel Adapter Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  error_recovery_enabled:
    description:
      - Enables Fibre Channel Error recovery.
    type: bool
    default: false
  port_down_timeout:
    description:
      - The number of milliseconds a remote Fibre Channel port should be offline before informing the SCSI upper layer.
      - This determines when the port is considered unavailable.
      - For a server with a VIC adapter running ESXi, the recommended value is 10000.
      - For a server with a port used to boot a Windows OS from the SAN, the recommended value is 5000 milliseconds.
    type: int
    default: 10000
  io_retry_timeout:
    description:
      - The number of seconds the adapter waits before aborting the pending command.
      - After timeout, it resends the same IO request.
    type: int
    default: 5
  link_down_timeout:
    description:
      - The number of milliseconds the port should actually be down before it is marked down.
      - After this timeout, fabric connectivity is considered lost.
    type: int
    default: 30000
  port_down_io_retry_count:
    description:
      - The number of times an I/O request to a port is retried because the port is busy.
      - After this count, the system decides the port is unavailable.
    type: int
    default: 8
  error_detection_timeout:
    description:
      - Error Detection Timeout, also referred to as EDTOV, is the number of milliseconds to wait.
      - This is the time before the system assumes that an error has occurred.
    type: int
    default: 2000
  resource_allocation_timeout:
    description:
      - Resource Allocation Timeout, also referred to as RATOV, is the number of milliseconds to wait.
      - This is the time before the system assumes that a resource cannot be properly allocated.
    type: int
    default: 10000
  flogi_retries:
    description:
      - The number of times that the system tries to log in to the fabric after the first failure.
    type: int
    default: 8
  flogi_timeout:
    description:
      - The number of milliseconds that the system waits before it tries to log in again.
    type: int
    default: 4000
  plogi_retries:
    description:
      - The number of times that the system tries to log in to a port after the first failure.
    type: int
    default: 8
  plogi_timeout:
    description:
      - The number of milliseconds that the system waits before it tries to log in again.
    type: int
    default: 20000
  interrupt_mode:
    description:
      - The preferred driver interrupt mode.
      - MSIx is the recommended option.
    type: str
    choices: [MSIx, MSI, INTx]
    default: MSIx
  io_throttle_count:
    description:
      - The maximum number of data or control I/O operations that can be pending for the virtual interface at one time.
      - If this value is exceeded, the additional I/O operations wait in the queue.
      - They wait until the number of pending I/O operations decreases and the additional operations can be processed.
    type: int
    default: 512
  lun_count:
    description:
      - The maximum number of LUNs that the Fibre Channel driver will export or show.
      - The maximum number of LUNs is usually controlled by the operating system running on the server.
      - Lun Count value can exceed 1024 only for vHBA of type 'FC Initiator' and on servers having supported firmware version.
    type: int
    default: 1024
  lun_queue_depth:
    description:
      - The number of commands that the HBA can send and receive in a single transmission per LUN.
    type: int
    default: 20
  rx_ring_size:
    description:
      - The number of descriptors in each receive queue.
      - The maximum value for Receive queue is 2048.
    type: int
    default: 64
  tx_ring_size:
    description:
      - The number of descriptors in each transmit queue.
      - The maximum value for Transmit queue is 128.
    type: int
    default: 64
  scsi_io_queue_count:
    description:
      - The number of SCSI I/O queue resources the system should allocate.
    type: int
    default: 1
  scsi_io_ring_size:
    description:
      - The number of descriptors in each SCSI I/O queue.
    type: int
    default: 512
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Fibre Channel Adapter Policy with default settings
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-adapter-default"
    description: "Fibre Channel Adapter policy with default values"
    state: present

- name: Create Fibre Channel Adapter Policy with custom error recovery settings
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-adapter-error-recovery"
    description: "FC Adapter policy with error recovery enabled"
    error_recovery_enabled: true
    port_down_timeout: 5000
    io_retry_timeout: 8
    link_down_timeout: 20000
    port_down_io_retry_count: 10
    tags:
      - Key: Environment
        Value: Production
    state: present

- name: Create Fibre Channel Adapter Policy with custom queue settings
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-adapter-high-performance"
    description: "FC Adapter policy with optimized queue settings"
    io_throttle_count: 1024
    lun_count: 4096
    lun_queue_depth: 254
    rx_ring_size: 128
    tx_ring_size: 128
    scsi_io_queue_count: 8
    scsi_io_ring_size: 512
    state: present

- name: Create Fibre Channel Adapter Policy with custom timeout settings
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "Engineering"
    name: "fc-adapter-custom-timeouts"
    description: "FC Adapter policy with custom timeout values"
    error_detection_timeout: 5000
    resource_allocation_timeout: 15000
    flogi_retries: 12
    flogi_timeout: 8000
    plogi_retries: 12
    plogi_timeout: 30000
    state: present

- name: Create Fibre Channel Adapter Policy with MSI interrupt mode
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-adapter-msi-mode"
    description: "FC Adapter policy with MSI interrupt mode"
    interrupt_mode: MSI
    state: present

- name: Update Fibre Channel Adapter Policy description
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-adapter-default"
    description: "Updated FC Adapter policy description"
    state: present

- name: Delete Fibre Channel Adapter Policy
  cisco.intersight.intersight_fibre_channel_adapter_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "fc-adapter-default"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "fc-adapter-default",
        "ObjectType": "vnic.FcAdapterPolicy",
        "Moid": "1234567890abcdef12345678",
        "Description": "Fibre Channel Adapter policy with default values",
        "ErrorRecoverySettings": {
            "Enabled": false,
            "PortDownTimeout": 10000,
            "LinkDownTimeout": 30000,
            "IoRetryTimeout": 5,
            "IoRetryCount": 8
        },
        "ErrorDetectionTimeout": 2000,
        "ResourceAllocationTimeout": 10000,
        "FlogiSettings": {
            "Retries": 8,
            "Timeout": 4000
        },
        "PlogiSettings": {
            "Retries": 8,
            "Timeout": 20000
        },
        "InterruptSettings": {
            "Mode": "MSIx"
        },
        "IoThrottleCount": 512,
        "LunCount": 1024,
        "LunQueueDepth": 20,
        "RxQueueSettings": {
            "RingSize": 64
        },
        "TxQueueSettings": {
            "RingSize": 64
        },
        "ScsiQueueSettings": {
            "Count": 1,
            "RingSize": 512
        },
        "Organization": {
            "Moid": "abcdef1234567890abcdef12",
            "ObjectType": "organization.Organization"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_parameters(module):
    """
    Validate module parameters for Fibre Channel Adapter policy configuration.
    """
    if module.params['state'] != 'present':
        return
    # Validate port_down_timeout range (0-240000)
    port_down_timeout = module.params.get('port_down_timeout')
    if port_down_timeout is not None and (port_down_timeout < 0 or port_down_timeout > 240000):
        module.fail_json(msg="Parameter 'port_down_timeout' must be between 0 and 240000")
    # Validate io_retry_timeout range (1-59)
    io_retry_timeout = module.params.get('io_retry_timeout')
    if io_retry_timeout is not None and (io_retry_timeout < 1 or io_retry_timeout > 59):
        module.fail_json(msg="Parameter 'io_retry_timeout' must be between 1 and 59")
    # Validate link_down_timeout range (0-240000)
    link_down_timeout = module.params.get('link_down_timeout')
    if link_down_timeout is not None and (link_down_timeout < 0 or link_down_timeout > 240000):
        module.fail_json(msg="Parameter 'link_down_timeout' must be between 0 and 240000")
    # Validate port_down_io_retry_count range (0-255)
    port_down_io_retry_count = module.params.get('port_down_io_retry_count')
    if port_down_io_retry_count is not None and (port_down_io_retry_count < 0 or port_down_io_retry_count > 255):
        module.fail_json(msg="Parameter 'port_down_io_retry_count' must be between 0 and 255")
    # Validate error_detection_timeout range (1000-100000)
    error_detection_timeout = module.params.get('error_detection_timeout')
    if error_detection_timeout is not None and (error_detection_timeout < 1000 or error_detection_timeout > 100000):
        module.fail_json(msg="Parameter 'error_detection_timeout' must be between 1000 and 100000")
    # Validate resource_allocation_timeout range (5000-100000)
    resource_allocation_timeout = module.params.get('resource_allocation_timeout')
    if resource_allocation_timeout is not None and (resource_allocation_timeout < 5000 or resource_allocation_timeout > 100000):
        module.fail_json(msg="Parameter 'resource_allocation_timeout' must be between 5000 and 100000")
    # Validate flogi_retries range (0-4294967295)
    flogi_retries = module.params.get('flogi_retries')
    if flogi_retries is not None and (flogi_retries < 0 or flogi_retries > 4294967295):
        module.fail_json(msg="Parameter 'flogi_retries' must be between 0 and 4294967295")
    # Validate flogi_timeout range (1000-255000)
    flogi_timeout = module.params.get('flogi_timeout')
    if flogi_timeout is not None and (flogi_timeout < 1000 or flogi_timeout > 255000):
        module.fail_json(msg="Parameter 'flogi_timeout' must be between 1000 and 255000")
    # Validate plogi_retries range (0-255)
    plogi_retries = module.params.get('plogi_retries')
    if plogi_retries is not None and (plogi_retries < 0 or plogi_retries > 255):
        module.fail_json(msg="Parameter 'plogi_retries' must be between 0 and 255")
    # Validate plogi_timeout range (1000-255000)
    plogi_timeout = module.params.get('plogi_timeout')
    if plogi_timeout is not None and (plogi_timeout < 1000 or plogi_timeout > 255000):
        module.fail_json(msg="Parameter 'plogi_timeout' must be between 1000 and 255000")
    # Validate io_throttle_count range (1-1024)
    io_throttle_count = module.params.get('io_throttle_count')
    if io_throttle_count is not None and (io_throttle_count < 1 or io_throttle_count > 1024):
        module.fail_json(msg="Parameter 'io_throttle_count' must be between 1 and 1024")
    # Validate lun_count range (1-4096)
    lun_count = module.params.get('lun_count')
    if lun_count is not None and (lun_count < 1 or lun_count > 4096):
        module.fail_json(msg="Parameter 'lun_count' must be between 1 and 4096")
    # Validate lun_queue_depth range (1-254)
    lun_queue_depth = module.params.get('lun_queue_depth')
    if lun_queue_depth is not None and (lun_queue_depth < 1 or lun_queue_depth > 254):
        module.fail_json(msg="Parameter 'lun_queue_depth' must be between 1 and 254")
    # Validate rx_ring_size (>= 64)
    rx_ring_size = module.params.get('rx_ring_size')
    if rx_ring_size is not None and rx_ring_size < 64:
        module.fail_json(msg="Parameter 'rx_ring_size' must be at least 64")
    # Validate tx_ring_size (>= 64)
    tx_ring_size = module.params.get('tx_ring_size')
    if tx_ring_size is not None and tx_ring_size < 64:
        module.fail_json(msg="Parameter 'tx_ring_size' must be at least 64")
    # Validate scsi_io_queue_count range (1-245)
    scsi_io_queue_count = module.params.get('scsi_io_queue_count')
    if scsi_io_queue_count is not None and (scsi_io_queue_count < 1 or scsi_io_queue_count > 245):
        module.fail_json(msg="Parameter 'scsi_io_queue_count' must be between 1 and 245")
    # Validate scsi_io_ring_size range (64-512)
    scsi_io_ring_size = module.params.get('scsi_io_ring_size')
    if scsi_io_ring_size is not None and (scsi_io_ring_size < 64 or scsi_io_ring_size > 512):
        module.fail_json(msg="Parameter 'scsi_io_ring_size' must be between 64 and 512")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        error_recovery_enabled=dict(type='bool', default=False),
        port_down_timeout=dict(type='int', default=10000),
        io_retry_timeout=dict(type='int', default=5),
        link_down_timeout=dict(type='int', default=30000),
        port_down_io_retry_count=dict(type='int', default=8),
        error_detection_timeout=dict(type='int', default=2000),
        resource_allocation_timeout=dict(type='int', default=10000),
        flogi_retries=dict(type='int', default=8),
        flogi_timeout=dict(type='int', default=4000),
        plogi_retries=dict(type='int', default=8),
        plogi_timeout=dict(type='int', default=20000),
        interrupt_mode=dict(type='str', choices=['MSIx', 'MSI', 'INTx'], default='MSIx'),
        io_throttle_count=dict(type='int', default=512),
        lun_count=dict(type='int', default=1024),
        lun_queue_depth=dict(type='int', default=20),
        rx_ring_size=dict(type='int', default=64),
        tx_ring_size=dict(type='int', default=64),
        scsi_io_queue_count=dict(type='int', default=1),
        scsi_io_ring_size=dict(type='int', default=512),
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Validate module parameters
    validate_parameters(module)

    # Resource path used to configure policy
    resource_path = '/vnic/FcAdapterPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name'],
    }

    if module.params['state'] == 'present':
        intersight.api_body['ErrorRecoverySettings'] = {
            'Enabled': module.params['error_recovery_enabled'],
            'PortDownTimeout': module.params['port_down_timeout'],
            'LinkDownTimeout': module.params['link_down_timeout'],
            'IoRetryTimeout': module.params['io_retry_timeout'],
            'IoRetryCount': module.params['port_down_io_retry_count'],
        }
        intersight.api_body['ErrorDetectionTimeout'] = module.params['error_detection_timeout']
        intersight.api_body['ResourceAllocationTimeout'] = module.params['resource_allocation_timeout']
        intersight.api_body['FlogiSettings'] = {
            'Retries': module.params['flogi_retries'],
            'Timeout': module.params['flogi_timeout'],
        }
        intersight.api_body['PlogiSettings'] = {
            'Retries': module.params['plogi_retries'],
            'Timeout': module.params['plogi_timeout'],
        }
        intersight.api_body['InterruptSettings'] = {
            'Mode': module.params['interrupt_mode'],
        }
        intersight.api_body['IoThrottleCount'] = module.params['io_throttle_count']
        intersight.api_body['LunCount'] = module.params['lun_count']
        intersight.api_body['LunQueueDepth'] = module.params['lun_queue_depth']
        intersight.api_body['RxQueueSettings'] = {
            'RingSize': module.params['rx_ring_size'],
        }
        intersight.api_body['TxQueueSettings'] = {
            'RingSize': module.params['tx_ring_size'],
        }
        intersight.api_body['ScsiQueueSettings'] = {
            'Count': module.params['scsi_io_queue_count'],
            'RingSize': module.params['scsi_io_ring_size'],
        }
        intersight.set_tags_and_description()
    intersight.configure_policy_or_profile(resource_path=resource_path)
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_settings
short_description: Resource module for networks _settings
description:
  - Manage operation update of the resource networks _settings.
  - Update the settings for a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  localStatusPage:
    description: A hash of Local Status page(s)' authentication options applied to
      the Network.
    suboptions:
      authentication:
        description: A hash of Local Status page(s)' authentication options applied
          to the Network.
        suboptions:
          enabled:
            description: Enables / disables the authentication on Local Status page(s).
            type: bool
          password:
            description: The password used for Local Status Page(s). Set this to null
              to clear the password.
            type: str
          username:
            description: The username used for Local Status Page(s). Setting this
              to null or empty string will clear the value, allowing you to use the
              default of 'admin'.
            type: str
        type: dict
    type: dict
  localStatusPageEnabled:
    description: Enables / disables the local device status pages (<a target='_blank'
      href='http //my.meraki.com/'>my.meraki.com, </a><a target='_blank' href='http
      //ap.meraki.com/'>ap.meraki.com, </a><a target='_blank' href='http //switch.meraki.com/'>switch.meraki.com,
      </a><a target='_blank' href='http //wired.meraki.com/'>wired.meraki.com</a>).
      Optional (defaults to false).
    type: bool
  namedVlans:
    description: A hash of Named VLANs options applied to the Network.
    suboptions:
      enabled:
        description: Enables / disables Named VLANs on the Network.
        type: bool
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  remoteStatusPageEnabled:
    description: Enables / disables access to the device status page (<a target='_blank'>http
      //device's LAN IP)</a>. Optional. Can only be set if localStatusPageEnabled
      is set to true.
    type: bool
  securePort:
    description: A hash of SecureConnect options applied to the Network.
    suboptions:
      enabled:
        description: Enables / disables SecureConnect on the network. Optional.
        type: bool
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks updateNetworkSettings
    description: Complete reference of the updateNetworkSettings API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-settings
notes:
  - SDK Method used are
    networks.Networks.update_network_settings,
  - Paths used are
    put /networks/{networkId}/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_settings:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    localStatusPage:
      authentication:
        enabled: false
        password: miles123
        username: admin
    localStatusPageEnabled: true
    namedVlans:
      enabled: true
    networkId: string
    remoteStatusPageEnabled: true
    securePort:
      enabled: false
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "localStatusPageEnabled": true,
      "remoteStatusPageEnabled": true,
      "localStatusPage": {
        "authentication": {
          "enabled": true,
          "username": "string"
        }
      },
      "securePort": {
        "enabled": true
      },
      "fips": {
        "enabled": true
      },
      "namedVlans": {
        "enabled": true
      }
    }
"""



#!/usr/bin/python

# Copyright (c) 2017, Kenneth D. Evensen <kdevensen@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pamd
author:
  - Kenneth D. Evensen (@kevensen)
short_description: Manage PAM Modules
description:
  - Edit PAM service's type, control, module path and module arguments.
  - In order for a PAM rule to be modified, the type, control and module_path must match an existing rule. See man(5) pam.d
    for details.
notes:
  - This module does not handle authselect profiles.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - The name generally refers to the PAM service file to change, for example system-auth.
    type: str
    required: true
  type:
    description:
      - The type of the PAM rule being modified.
      - The O(type), O(control), and O(module_path) options all must match a rule to be modified.
    type: str
    required: true
    choices: [account, -account, auth, -auth, password, -password, session, -session]
  control:
    description:
      - The control of the PAM rule being modified.
      - This may be a complicated control with brackets. If this is the case, be sure to put "[bracketed controls]" in quotes.
      - The O(type), O(control), and O(module_path) options all must match a rule to be modified.
    type: str
    required: true
  module_path:
    description:
      - The module path of the PAM rule being modified.
      - The O(type), O(control), and O(module_path) options all must match a rule to be modified.
    type: str
    required: true
  new_type:
    description:
      - The new type to assign to the new rule.
    type: str
    choices: [account, -account, auth, -auth, password, -password, session, -session]
  new_control:
    description:
      - The new control to assign to the new rule.
    type: str
  new_module_path:
    description:
      - The new module path to be assigned to the new rule.
    type: str
  module_arguments:
    description:
      - When O(state=updated), the O(module_arguments) replace existing module_arguments.
      - When O(state=args_absent) args matching those listed in O(module_arguments) are removed.
      - When O(state=args_present) any args listed in O(module_arguments) are added if missing from the existing rule.
      - Furthermore, if the module argument takes a value denoted by C(=), the value changes to that specified in module_arguments.
    type: list
    elements: str
  state:
    description:
      - The default of V(updated) modifies an existing rule if type, control and module_path all match an existing rule.
      - With V(before), the new rule is inserted before a rule matching type, control and module_path.
      - Similarly, with V(after), the new rule is inserted after an existing rulematching type, control and module_path.
      - With either V(before) or V(after) O(new_type), O(new_control), and O(new_module_path) must all be specified.
      - If state is V(args_absent) or V(args_present), O(new_type), O(new_control), and O(new_module_path) are ignored.
      - State V(absent) removes the rule.
    type: str
    choices: [absent, before, after, args_absent, args_present, updated]
    default: updated
  path:
    description:
      - This is the path to the PAM service files.
    type: path
    default: /etc/pam.d
  backup:
    description:
      - Create a backup file including the timestamp information so you can get the original file back if you somehow clobbered
        it incorrectly.
    type: bool
    default: false
"""

EXAMPLES = r"""
- name: Update pamd rule's control in /etc/pam.d/system-auth
  community.general.pamd:
    name: system-auth
    type: auth
    control: required
    module_path: pam_faillock.so
    new_control: sufficient

- name: Update pamd rule's complex control in /etc/pam.d/system-auth
  community.general.pamd:
    name: system-auth
    type: session
    control: '[success=1 default=ignore]'
    module_path: pam_succeed_if.so
    new_control: '[success=2 default=ignore]'

- name: Insert a new rule before an existing rule
  community.general.pamd:
    name: system-auth
    type: auth
    control: required
    module_path: pam_faillock.so
    new_type: auth
    new_control: sufficient
    new_module_path: pam_faillock.so
    state: before

- name: Insert a new rule pam_wheel.so with argument 'use_uid' after an existing rule pam_rootok.so
  community.general.pamd:
    name: su
    type: auth
    control: sufficient
    module_path: pam_rootok.so
    new_type: auth
    new_control: required
    new_module_path: pam_wheel.so
    module_arguments: 'use_uid'
    state: after

- name: Remove module arguments from an existing rule
  community.general.pamd:
    name: system-auth
    type: auth
    control: required
    module_path: pam_faillock.so
    module_arguments: ''
    state: updated

- name: Replace all module arguments in an existing rule
  community.general.pamd:
    name: system-auth
    type: auth
    control: required
    module_path: pam_faillock.so
    module_arguments: 'preauth silent deny=3 unlock_time=604800 fail_interval=900'
    state: updated

- name: Remove specific arguments from a rule
  community.general.pamd:
    name: system-auth
    type: session
    control: '[success=1 default=ignore]'
    module_path: pam_succeed_if.so
    module_arguments: crond,quiet
    state: args_absent

- name: Ensure specific arguments are present in a rule
  community.general.pamd:
    name: system-auth
    type: session
    control: '[success=1 default=ignore]'
    module_path: pam_succeed_if.so
    module_arguments: crond,quiet
    state: args_present

- name: Ensure specific arguments are present in a rule (alternative)
  community.general.pamd:
    name: system-auth
    type: session
    control: '[success=1 default=ignore]'
    module_path: pam_succeed_if.so
    module_arguments:
      - crond
      - quiet
    state: args_present

- name: Module arguments requiring commas must be listed as a Yaml list
  community.general.pamd:
    name: special-module
    type: account
    control: required
    module_path: pam_access.so
    module_arguments:
      - listsep=,
    state: args_present

- name: Update specific argument value in a rule
  community.general.pamd:
    name: system-auth
    type: auth
    control: required
    module_path: pam_faillock.so
    module_arguments: 'fail_interval=300'
    state: args_present

- name: Add pam common-auth rule for duo
  community.general.pamd:
    name: common-auth
    new_type: auth
    new_control: '[success=1 default=ignore]'
    new_module_path: '/lib64/security/pam_duo.so'
    state: after
    type: auth
    module_path: pam_sss.so
    control: 'requisite'
"""

RETURN = r"""
change_count:
  description: How many rules were changed.
  type: int
  sample: 1
  returned: success
backupdest:
  description:
    - The file name of the backup file, if created.
  returned: success
  type: str
"""


import os
import re
from datetime import datetime
from tempfile import NamedTemporaryFile

from ansible.module_utils.basic import AnsibleModule

RULE_REGEX = re.compile(
    r"""(?P<rule_type>-?(?:auth|account|session|password))\s+
                        (?P<control>\[.*\]|\S*)\s+
                        (?P<path>\S*)\s*
                        (?P<args>.*)\s*""",
    re.X,
)
RULE_ARG_REGEX = re.compile(r"(\[.*\]|\S*)")

VALID_TYPES = ["account", "-account", "auth", "-auth", "password", "-password", "session", "-session"]


class PamdLine:
    def __init__(self, line):
        self.line = line
        self.prev = None
        self.next = None

    @property
    def is_valid(self):
        return self.line.strip() == ""

    def validate(self):
        if not self.is_valid:
            return False, f"Rule is not valid {self.line}"
        return True, f"Rule is valid {self.line}"

    # Method to check if a rule matches the type, control and path.
    def matches(self, rule_type, rule_control, rule_path, rule_args=None):
        return False

    def __str__(self):
        return str(self.line)


class PamdEmptyLine(PamdLine):
    pass


class PamdComment(PamdLine):
    def __init__(self, line):
        super().__init__(line)

    @property
    def is_valid(self):
        return self.line.startswith("#")


class PamdInclude(PamdLine):
    def __init__(self, line):
        super().__init__(line)

    @property
    def is_valid(self):
        return self.line.startswith("@include")


class PamdRule(PamdLine):
    valid_simple_controls = ["required", "requisite", "sufficient", "optional", "include", "substack", "definitive"]
    valid_control_values = [
        "success",
        "open_err",
        "symbol_err",
        "service_err",
        "system_err",
        "buf_err",
        "perm_denied",
        "auth_err",
        "cred_insufficient",
        "authinfo_unavail",
        "user_unknown",
        "maxtries",
        "new_authtok_reqd",
        "acct_expired",
        "session_err",
        "cred_unavail",
        "cred_expired",
        "cred_err",
        "no_module_data",
        "conv_err",
        "authtok_err",
        "authtok_recover_err",
        "authtok_lock_busy",
        "authtok_disable_aging",
        "try_again",
        "ignore",
        "abort",
        "authtok_expired",
        "module_unknown",
        "bad_item",
        "conv_again",
        "incomplete",
        "default",
    ]
    valid_control_actions = ["ignore", "bad", "die", "ok", "done", "reset"]

    def __init__(self, rule_type, rule_control, rule_path, rule_args=None):
        self.prev = None
        self.next = None
        self._control = None
        self._args = None
        self.rule_type = rule_type
        self.rule_control = rule_control

        self.rule_path = rule_path
        self.rule_args = rule_args

    # Method to check if a rule matches the type, control and path.
    def matches(self, rule_type, rule_control, rule_path, rule_args=None):
        return rule_type == self.rule_type and rule_control == self.rule_control and rule_path == self.rule_path

    @classmethod
    def rule_from_string(cls, line):
        rule_match = RULE_REGEX.search(line)
        if rule_match is None:
            return None
        rule_args = parse_module_arguments(rule_match.group("args"))
        return cls(rule_match.group("rule_type"), rule_match.group("control"), rule_match.group("path"), rule_args)

    def __str__(self):
        if self.rule_args:
            return f"{self.rule_type: <11}{self.rule_control} {self.rule_path} {' '.join(self.rule_args)}"
        return f"{self.rule_type: <11}{self.rule_control} {self.rule_path}"

    @property
    def rule_control(self):
        if isinstance(self._control, list):
            return f"[{' '.join(self._control)}]"
        return self._control

    @rule_control.setter
    def rule_control(self, control):
        if control.startswith("["):
            control = control.replace(" = ", "=").replace("[", "").replace("]", "")
            self._control = control.split(" ")
        else:
            self._control = control

    @property
    def rule_args(self):
        if not self._args:
            return []
        return self._args

    @rule_args.setter
    def rule_args(self, args):
        self._args = parse_module_arguments(args)

    @property
    def line(self):
        return str(self)

    @classmethod
    def is_action_unsigned_int(cls, string_num):
        number = 0
        try:
            number = int(string_num)
        except ValueError:
            return False

        return number >= 0

    @property
    def is_valid(self):
        return self.validate()[0]

    def validate(self):
        # Validate the rule type
        if self.rule_type not in VALID_TYPES:
            return False, f"Rule type, {self.rule_type}, is not valid in rule {self.line}"
        # Validate the rule control
        if isinstance(self._control, str) and self.rule_control not in PamdRule.valid_simple_controls:
            return False, f"Rule control, {self.rule_control}, is not valid in rule {self.line}"
        elif isinstance(self._control, list):
            for control in self._control:
                value, action = control.split("=")
                if value not in PamdRule.valid_control_values:
                    return False, f"Rule control value, {value}, is not valid in rule {self.line}"
                if action not in PamdRule.valid_control_actions and not PamdRule.is_action_unsigned_int(action):
                    return False, f"Rule control action, {action}, is not valid in rule {self.line}"

        # TODO: Validate path

        return True, f"Rule is valid {self.line}"


# PamdService encapsulates an entire service and contains one or more rules.  It seems the best way is to do this
# as a doubly linked list.
class PamdService:
    def __init__(self, content):
        self._head = None
        self._tail = None
        for line in content.splitlines():
            if line.lstrip().startswith("#"):
                pamd_line = PamdComment(line)
            elif line.lstrip().startswith("@include"):
                pamd_line = PamdInclude(line)
            elif line.strip() == "":
                pamd_line = PamdEmptyLine(line)
            else:
                pamd_line = PamdRule.rule_from_string(line) or PamdLine(line)

            self.append(pamd_line)

    def append(self, pamd_line):
        if self._head is None:
            self._head = self._tail = pamd_line
        else:
            pamd_line.prev = self._tail
            pamd_line.next = None
            self._tail.next = pamd_line
            self._tail = pamd_line

    def remove(self, rule_type, rule_control, rule_path):
        current_line = self._head
        changed = 0

        while current_line is not None:
            if current_line.matches(rule_type, rule_control, rule_path):
                if current_line.prev is not None:
                    current_line.prev.next = current_line.next
                    if current_line.next is not None:
                        current_line.next.prev = current_line.prev
                else:
                    self._head = current_line.next
                    current_line.next.prev = None
                changed += 1

            current_line = current_line.next
        return changed

    def get(self, rule_type, rule_control, rule_path):
        lines = []
        current_line = self._head
        while current_line is not None:
            if isinstance(current_line, PamdRule) and current_line.matches(rule_type, rule_control, rule_path):
                lines.append(current_line)

            current_line = current_line.next

        return lines

    def has_rule(self, rule_type, rule_control, rule_path):
        return bool(self.get(rule_type, rule_control, rule_path))

    def update_rule(
        self, rule_type, rule_control, rule_path, new_type=None, new_control=None, new_path=None, new_args=None
    ):
        # Get a list of rules we want to change
        rules_to_find = self.get(rule_type, rule_control, rule_path)

        new_args = parse_module_arguments(new_args, return_none=True)

        changes = 0
        for current_rule in rules_to_find:
            rule_changed = False
            if new_type:
                if current_rule.rule_type != new_type:
                    rule_changed = True
                    current_rule.rule_type = new_type
            if new_control:
                if current_rule.rule_control != new_control:
                    rule_changed = True
                    current_rule.rule_control = new_control
            if new_path:
                if current_rule.rule_path != new_path:
                    rule_changed = True
                    current_rule.rule_path = new_path
            if new_args is not None:
                if current_rule.rule_args != new_args:
                    rule_changed = True
                    current_rule.rule_args = new_args

            if rule_changed:
                changes += 1

        return changes

    def insert_before(
        self, rule_type, rule_control, rule_path, new_type=None, new_control=None, new_path=None, new_args=None
    ):
        # Get a list of rules we want to change
        rules_to_find = self.get(rule_type, rule_control, rule_path)
        changes = 0
        # There are two cases to consider.
        # 1. The new rule doesn't exist before the existing rule
        # 2. The new rule exists

        for current_rule in rules_to_find:
            # Create a new rule
            new_rule = PamdRule(new_type, new_control, new_path, new_args)
            # First we'll get the previous rule.
            previous_rule = current_rule.prev

            # Next we may have to loop backwards if the previous line is a comment.  If it
            # is, we'll get the previous "rule's" previous.
            while previous_rule is not None and isinstance(previous_rule, (PamdComment, PamdEmptyLine)):
                previous_rule = previous_rule.prev
            # Next we'll see if the previous rule matches what we are trying to insert.
            if previous_rule is not None and not previous_rule.matches(new_type, new_control, new_path):
                # First set the original previous rule's next to the new_rule
                previous_rule.next = new_rule
                # Second, set the new_rule's previous to the original previous
                new_rule.prev = previous_rule
                # Third, set the new rule's next to the current rule
                new_rule.next = current_rule
                # Fourth, set the current rule's previous to the new_rule
                current_rule.prev = new_rule

                changes += 1

            # Handle the case where it is the first rule in the list.
            elif previous_rule is None:
                # This is the case where the current rule is not only the first rule
                # but the first line as well.  So we set the head to the new rule
                if current_rule.prev is None:
                    self._head = new_rule
                # This case would occur if the previous line was a comment.
                else:
                    current_rule.prev.next = new_rule
                new_rule.prev = current_rule.prev
                new_rule.next = current_rule
                current_rule.prev = new_rule
                changes += 1

        return changes

    def insert_after(
        self, rule_type, rule_control, rule_path, new_type=None, new_control=None, new_path=None, new_args=None
    ):
        # Get a list of rules we want to change
        rules_to_find = self.get(rule_type, rule_control, rule_path)
        changes = 0
        # There are two cases to consider.
        # 1. The new rule doesn't exist after the existing rule
        # 2. The new rule exists
        for current_rule in rules_to_find:
            # First we'll get the next rule.
            next_rule = current_rule.next
            # Next we may have to loop forwards if the next line is a comment.  If it
            # is, we'll get the next "rule's" next.
            while next_rule is not None and isinstance(next_rule, (PamdComment, PamdEmptyLine)):
                next_rule = next_rule.next

            # First we create a new rule
            new_rule = PamdRule(new_type, new_control, new_path, new_args)
            if next_rule is not None and not next_rule.matches(new_type, new_control, new_path):
                # If the previous rule doesn't match we'll insert our new rule.

                # Second set the original next rule's previous to the new_rule
                next_rule.prev = new_rule
                # Third, set the new_rule's next to the original next rule
                new_rule.next = next_rule
                # Fourth, set the new rule's previous to the current rule
                new_rule.prev = current_rule
                # Fifth, set the current rule's next to the new_rule
                current_rule.next = new_rule

                changes += 1

            # This is the case where the current_rule is the last in the list
            elif next_rule is None:
                new_rule.prev = self._tail
                new_rule.next = None
                self._tail.next = new_rule
                self._tail = new_rule

                current_rule.next = new_rule
                changes += 1

        return changes

    def add_module_arguments(self, rule_type, rule_control, rule_path, args_to_add):
        # Get a list of rules we want to change
        rules_to_find = self.get(rule_type, rule_control, rule_path)

        args_to_add = parse_module_arguments(args_to_add)

        changes = 0

        for current_rule in rules_to_find:
            rule_changed = False

            # create some structures to evaluate the situation
            simple_new_args = set()
            key_value_new_args = dict()

            for arg in args_to_add:
                if arg.startswith("["):
                    continue
                elif "=" in arg:
                    key, value = arg.split("=")
                    key_value_new_args[key] = value
                else:
                    simple_new_args.add(arg)

            key_value_new_args_set = set(key_value_new_args)

            simple_current_args = set()
            key_value_current_args = dict()

            for arg in current_rule.rule_args:
                if arg.startswith("["):
                    continue
                elif "=" in arg:
                    key, value = arg.split("=")
                    key_value_current_args[key] = value
                else:
                    simple_current_args.add(arg)

            key_value_current_args_set = set(key_value_current_args)

            new_args_to_add = list()

            # Handle new simple arguments
            if simple_new_args.difference(simple_current_args):
                for arg in simple_new_args.difference(simple_current_args):
                    new_args_to_add.append(arg)

            # Handle new key value arguments
            if key_value_new_args_set.difference(key_value_current_args_set):
                for key in key_value_new_args_set.difference(key_value_current_args_set):
                    new_args_to_add.append(f"{key}={key_value_new_args[key]}")

            if new_args_to_add:
                current_rule.rule_args += new_args_to_add
                rule_changed = True

            # Handle existing key value arguments when value is not equal
            if key_value_new_args_set.intersection(key_value_current_args_set):
                for key in key_value_new_args_set.intersection(key_value_current_args_set):
                    if key_value_current_args[key] != key_value_new_args[key]:
                        arg_index = current_rule.rule_args.index(f"{key}={key_value_current_args[key]}")
                        current_rule.rule_args[arg_index] = str(f"{key}={key_value_new_args[key]}")
                        rule_changed = True

            if rule_changed:
                changes += 1

        return changes

    def remove_module_arguments(self, rule_type, rule_control, rule_path, args_to_remove):
        # Get a list of rules we want to change
        rules_to_find = self.get(rule_type, rule_control, rule_path)

        args_to_remove = parse_module_arguments(args_to_remove)

        changes = 0

        for current_rule in rules_to_find:
            if not args_to_remove:
                args_to_remove = []

            # Let's check to see if there are any args to remove by finding the intersection
            # of the rule's current args and the args_to_remove lists
            if not list(set(current_rule.rule_args) & set(args_to_remove)):
                continue

            # There are args to remove, so we create a list of new_args absent the args
            # to remove.
            current_rule.rule_args = [arg for arg in current_rule.rule_args if arg not in args_to_remove]

            changes += 1

        return changes

    def validate(self):
        current_line = self._head

        while current_line is not None:
            curr_validate = current_line.validate()
            if not curr_validate[0]:
                return curr_validate
            current_line = current_line.next
        return True, "Module is valid"

    def __str__(self):
        lines = []
        current_line = self._head

        mark = f"# Updated by Ansible - {datetime.now().isoformat()}"
        while current_line is not None:
            lines.append(str(current_line))
            current_line = current_line.next

        if len(lines) <= 1:
            lines.insert(0, "")
            lines.insert(1, mark)
        else:
            if lines[1].startswith("# Updated by Ansible"):
                lines[1] = mark
            else:
                lines.insert(1, mark)

        lines_joined = "\n".join(lines)
        return f"{lines_joined}\n"


def parse_module_arguments(module_arguments, return_none=False):
    # If args is None, return empty list by default.
    # But if return_none is True, then return None
    if module_arguments is None:
        return None if return_none else []
    if isinstance(module_arguments, list) and len(module_arguments) == 1 and not module_arguments[0]:
        return []

    if not isinstance(module_arguments, list):
        module_arguments = [module_arguments]

    # From this point on, module_arguments is guaranteed to be a list, empty or not
    parsed_args = []

    re_clear_spaces = re.compile(r"\s*=\s*")
    for arg in module_arguments:
        for item in filter(None, RULE_ARG_REGEX.findall(arg)):
            if not item.startswith("["):
                re_clear_spaces.sub("=", item)
            parsed_args.append(item)

    return parsed_args


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            type=dict(type="str", required=True, choices=VALID_TYPES),
            control=dict(type="str", required=True),
            module_path=dict(type="str", required=True),
            new_type=dict(type="str", choices=VALID_TYPES),
            new_control=dict(type="str"),
            new_module_path=dict(type="str"),
            module_arguments=dict(type="list", elements="str"),
            state=dict(
                type="str",
                default="updated",
                choices=["absent", "after", "args_absent", "args_present", "before", "updated"],
            ),
            path=dict(type="path", default="/etc/pam.d"),
            backup=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
        required_if=[
            ("state", "args_present", ["module_arguments"]),
            ("state", "args_absent", ["module_arguments"]),
            ("state", "before", ["new_control", "new_type", "new_module_path"]),
            ("state", "after", ["new_control", "new_type", "new_module_path"]),
        ],
    )
    content = ""
    fname = os.path.join(module.params["path"], module.params["name"])

    # Open the file and read the content or fail
    try:
        with open(fname) as service_file_obj:
            content = service_file_obj.read()
    except OSError as e:
        # If unable to read the file, fail out
        module.fail_json(msg=f"Unable to open/read PAM module file {fname} with error {e}.")

    # Assuming we didn't fail, create the service
    service = PamdService(content)
    # Set the action
    action = module.params["state"]

    changes = 0

    # Take action
    if action == "updated":
        changes = service.update_rule(
            module.params["type"],
            module.params["control"],
            module.params["module_path"],
            module.params["new_type"],
            module.params["new_control"],
            module.params["new_module_path"],
            module.params["module_arguments"],
        )
    elif action == "before":
        changes = service.insert_before(
            module.params["type"],
            module.params["control"],
            module.params["module_path"],
            module.params["new_type"],
            module.params["new_control"],
            module.params["new_module_path"],
            module.params["module_arguments"],
        )
    elif action == "after":
        changes = service.insert_after(
            module.params["type"],
            module.params["control"],
            module.params["module_path"],
            module.params["new_type"],
            module.params["new_control"],
            module.params["new_module_path"],
            module.params["module_arguments"],
        )
    elif action == "args_absent":
        changes = service.remove_module_arguments(
            module.params["type"],
            module.params["control"],
            module.params["module_path"],
            module.params["module_arguments"],
        )
    elif action == "args_present":
        if [arg for arg in parse_module_arguments(module.params["module_arguments"]) if arg.startswith("[")]:
            module.fail_json(
                msg="Unable to process bracketed '[' complex arguments with 'args_present'. Please use 'updated'."
            )

        changes = service.add_module_arguments(
            module.params["type"],
            module.params["control"],
            module.params["module_path"],
            module.params["module_arguments"],
        )
    elif action == "absent":
        changes = service.remove(module.params["type"], module.params["control"], module.params["module_path"])

    valid, msg = service.validate()

    # If the module is not valid (meaning one of the rules is invalid), we will fail
    if not valid:
        module.fail_json(msg=msg)

    result = dict(
        changed=(changes > 0),
        change_count=changes,
        backupdest="",
    )

    # If not check mode and something changed, backup the original if necessary then write out the file or fail
    if not module.check_mode and result["changed"]:
        # First, create a backup if desired.
        if module.params["backup"]:
            result["backupdest"] = module.backup_local(fname)
        try:
            temp_file = NamedTemporaryFile(mode="w", dir=module.tmpdir, delete=False)
            with open(temp_file.name, "w") as fd:
                fd.write(str(service))

        except OSError:
            module.fail_json(msg=f"Unable to create temporary file {temp_file}")

        module.atomic_move(temp_file.name, os.path.realpath(fname))

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_stp
short_description: Resource module for networks _switch _stp
description:
  - Manage operation update of the resource networks _switch _stp.
  - Updates STP settings.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  rstpEnabled:
    description: The spanning tree protocol status in network.
    type: bool
  stpBridgePriority:
    description: STP bridge priority for switches/stacks or switch templates. An empty
      array will clear the STP bridge priority settings.
    elements: dict
    suboptions:
      stacks:
        description: List of stack IDs.
        elements: str
        type: list
      stpPriority:
        description: STP priority for switch, stacks, or switch templates.
        type: int
      switchProfiles:
        description: List of switch template IDs.
        elements: str
        type: list
      switches:
        description: List of switch serial numbers.
        elements: str
        type: list
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch updateNetworkSwitchStp
    description: Complete reference of the updateNetworkSwitchStp API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-stp
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_stp,
  - Paths used are
    put /networks/{networkId}/switch/stp,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_stp:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    networkId: string
    rstpEnabled: true
    stpBridgePriority:
      - stacks:
          - '789102'
          - '123456'
          - '129102'
        stpPriority: 4096
        switchProfiles:
          - '1098'
          - '1099'
          - '1100'
        switches:
          - Q234-ABCD-0001
          - Q234-ABCD-0002
          - Q234-ABCD-0003
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "rstpEnabled": true,
      "stpBridgePriority": [
        {
          "switches": [
            "string"
          ],
          "stacks": [
            "string"
          ],
          "switchProfiles": [
            "string"
          ],
          "stpPriority": 0
        }
      ]
    }
"""
